@@ -45,69 +45,35 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
   int   status;
 
   base_single = cmatrix(0, num_residue, 0, 120);
-  base_sugar = cmatrix(0, num_residue, 0, 120);
-  base_p = cmatrix(0, num_residue, 0, 120);
-  other = cmatrix(0, num_residue, 0, 120);
+  base_sugar  = cmatrix(0, num_residue, 0, 120);
+  base_p      = cmatrix(0, num_residue, 0, 120);
+  other       = cmatrix(0, num_residue, 0, 120);
   
   hb_atom1 = cmatrix(0, BUF512, 0, 4);
   hb_atom2 = cmatrix(0, BUF512, 0, 4);
-  hb_dist = dvector(0, BUF512);
+  hb_dist  = dvector(0, BUF512);
   
   hb_crt_alt(HB_UPPER, HB_ATOM, b1);
   
-  pair_info = lmatrix(0, num_residue, 0, NP); /* detailed base-pair network*/
+  pair_info = lmatrix(0, num_residue, 0, NPT); /* detailed base-pair network*/
   prot_rna = lvector(0, num_residue); 
   
-  for(i=0;i<=num_residue; i++)
-    for(j=0;j<=NP; j++)
+  for (i = 0; i <= num_residue; i++)
+    for (j = 0; j <= NPT; j++)
       pair_info[i][j]=0;
   
-  for(i=0;i<BUF512; i++){
-    strcpy(hb_atom1[i],"");
-    strcpy(hb_atom2[i],"");
+  for (i = 0; i < BUF512; i++){
+    strcpy(hb_atom1[i], "");
+    strcpy(hb_atom2[i], "");
     hb_dist[i]=0;
   }
   
-  
-  /*
-    fprintf(fout, "------------------------------------------------\n");    
-    fprintf(fout,"Base-pair criteria used: \n");
-    fprintf(fout, "%6.2f --> upper H-bond length limits (ON..ON).\n", BPRS[1]);    
-    
-    fprintf(fout, "%6.2f --> max. distance between paired base origins.\n",
-    BPRS[2]);
-    
-    fprintf(fout, "%6.2f --> max. vertical distance between paired"
-    " base origins.\n", BPRS[3]);    
-    fprintf(fout, "%6.2f --> max. angle between paired bases [0-90].\n",
-    BPRS[4]);
-    
-    fprintf(fout, "%6.2f --> MIN. distance between RN9/YN1 atoms.\n",
-    BPRS[5]);
-    fprintf(fout, "%6.2f --> max. distance criterion for helix break[0-12]\n",
-    BPRS[6]);
-    
-    fprintf(fout,"------------------------------------------------ \n");
-    fprintf(fout,"INSTRUCTIONS: \n");
-    
-    fprintf(fout,"Column 1, 2, 3 are Chain ID, Residue number, Residue name\n");
-    fprintf(fout,"W.C. pairs are annotated as -/- (AU,AT) or +/+ (GC)\n");
-    fprintf(fout,"The three edges: W(Watson-Crick); H(Hoogsteen); S(suger)\n");
-    fprintf(fout,"Glycosidic bond orientation is annotated as (cis or trans).\n");
-    fprintf(fout,"Syn sugar-base conformations are annotated as (syn).\n");
-    fprintf(fout,"Stacked base pairs are annotated as (stack).\n");
-    fprintf(fout,"Non-identified edges are annotated as (.) or (?)\n");
-    fprintf(fout,"Tertiary interactions are marked by (!) in the line.\n");
-    
-    fprintf(fout,"------------------------------------------------ \n");
-  */
-
   list = er_CreateList(alloc_np);
-  fprintf(fout,   "BEGIN_base-pair\n" );
+  fprintf(fout,  "BEGIN_base-pair\n" );
 
   for (i = 1; i <= num_residue; i++){   
-    prot_rna[i]=0; 
-    sugar_syn[i]=0;
+    prot_rna[i]  = 0; 
+    sugar_syn[i] = 0;
   }
   
   /* change this condition if used 
@@ -118,46 +84,44 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
   
   for (i = 1; i < num_residue; i++) {
 
-    ir = seidx[i][1];
+    ir         = seidx[i][1];
     ichain_idx = er_ChainIdx(ChainID[ir], nchain_tot, chain_name);
-    from = chain_f[ichain_idx];
+    from       = chain_f[ichain_idx];
    
    for (j = i + 1; j <= num_residue; j++) {
       
-      jr = seidx[j][1];
-      jchain_idx = er_ChainIdx(ChainID[jr], nchain_tot, chain_name);
-      
-      to = chain_t[jchain_idx];
+      jr         = seidx[j][1];
+      jchain_idx = er_ChainIdx(ChainID[jr], nchain_tot, chain_name);      
+      to         = chain_t[jchain_idx];
 
-      if(sugar_syn[i] ==1){
+      if (sugar_syn[i] == 1) {
 	/*
 	  sprintf(syn_i, "%c:%ld %3s ", ChainID[ir], ResSeq[ir],"syn");
 	*/
 	sprintf(syn_i, "%3s ", "syn");    
-      }else{
+      } else {
 	strcpy(syn_i,"  ");
       }
-      if(sugar_syn[j] ==1){
+      if (sugar_syn[j] == 1) {
 	sprintf(syn_j, "%3s ", "syn");
 	/*
 	  sprintf(syn_j, "%c:%ld %3s ", ChainID[jr], ResSeq[jr],"syn");
 	*/
-      }else{
-	strcpy(syn_j,"  ");
+      } else {
+	strcpy(syn_j, "  ");
       }
       
-      
       check_pairs(i, j, bseq, seidx, xyz, Nxyz, orien, org,
 		  AtomName,BPRS, rtn_val, &bpid_lu, 0);
       /* here at least one good NO H bond (donor and acceptor)*/
       
-      if(bpid_lu){ /* only base-base */
+      if (bpid_lu){ /* only base-base */
 	
 	ir = seidx[i][1];
 	jr = seidx[j][1];
 	sprintf(work_num, "%ld\t%ld", i, j);
         
-	change=0.35; /* change the value for new H bond dist!! */
+	change = 0.35; /* change the value for new H bond dist!! */
         
 	
 	/* the last two value 1, 0 is the followings :
@@ -166,8 +130,8 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	   bone_key= 0;
 	   a key wheather to include O3', O5', O1P,O2P;   0 not include 
 	*/
-	c_key=1;
-	bone_key=0;
+	c_key    = 1;
+	bone_key = 0;
 	Hbond_pair(i, j, seidx, AtomName, bseq, xyz, change,
 		   &nh, hb_atom1, hb_atom2, hb_dist, c_key, bone_key);                
 	
@@ -178,21 +142,21 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	
 	
 	/* test new H bonds after increasing H bond distance */
-	nnh=0; 
+	nnh = 0; 
 	for (k = 1; k <=nh ; k++) {
-	  if(hb_atom1[k][3] != ' ' && hb_atom2[k][3] != ' ')
+	  if (hb_atom1[k][3] != ' ' && hb_atom2[k][3] != ' ')
 	    continue;
 	  nnh++;
 	}
 	
-	if(nnh == 1){ /*single base-to-base H bond */
+	if (nnh == 1){ /*single base-to-base H bond */
 	  
-	  if(nb_single++ > num_residue-2){
+	  if (nb_single++ > num_residue-2){
 	    printf("increase memory for single H bond case\n");
 	    fprintf(fout,   "END_base-pair\n" );
 	    return eslFAIL;
 	  }
-	  geo_check=5.2; /*for edge to edge check, larger for single bond*/
+	  geo_check = 5.2; /*for edge to edge check, larger for single bond*/
           
 	  LW_pair_type(i, j, geo_check, seidx, AtomName, HB_ATOM, xyz,
 		       bseq, hb_atom1, hb_atom2, hb_dist, type);
@@ -211,9 +175,9 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	  free_cmatrix(atom, 0,nh, 0,4);
           
 	  /* bifurcated  bond  (also belongs to 12 families, but with more restraits)*/
-	  if((nh1==1 && nh2>1) || (nh2==1 && nh1>1)){
-	    if((rtn_val[2] > BPRS[3]-0.6) ||(rtn_val[3] > BPRS[4]-20)){
-	      if(nb_single++ > num_residue-2){
+	  if ((nh1==1 && nh2>1) || (nh2==1 && nh1>1)){
+	    if ((rtn_val[2] > BPRS[3]-0.6) ||(rtn_val[3] > BPRS[4]-20)){
+	      if (nb_single++ > num_residue-2){
 		fprintf(fout,"END_base-pair\n" );
 		printf("increase memory for single H bond case\n");
 		return eslFAIL;
@@ -261,18 +225,18 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	  }
 	  else {/* 2 H bonds */
 	    
-	    if(bpid_lu ==2){ /* W.C. cases */
+	    if (bpid_lu ==2){ /* W.C. cases */
 	      
-	      if( (toupper(bseq[i]) == 'A' &&
+	      if ( (toupper(bseq[i]) == 'A' &&
 		   (toupper(bseq[j]) == 'U' || toupper(bseq[j]) == 'T'))
 		  ||(toupper(bseq[j]) == 'A' &&
 		     (toupper(bseq[i]) == 'U' || toupper(bseq[i]) == 'T'))){
 		strcpy(type, "WWc");
 		
-	      }else if( (toupper(bseq[i]) == 'G' && toupper(bseq[j]) == 'C') ||
+	      } else if ( (toupper(bseq[i]) == 'G' && toupper(bseq[j]) == 'C') ||
 			(toupper(bseq[j]) == 'G' && toupper(bseq[i]) == 'C') ){  
 		strcpy(type, "WWc");
-	      }else {
+	      } else {
 		strcpy(type, "WWc"); /*CI or IC*/
 	      }
 	      
@@ -295,7 +259,7 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 
 	  fprintf(fout, "%9s\t%c: %5ld %c %c %5ld %c: %7s %c %s%s %s\n",
 		  work_num,
-		  ChainID[ir], ResSeq[ir], bseq[i], bseq[j],ResSeq[jr],
+		  ChainID[ir], ResSeq[ir], bseq[i], bseq[j], ResSeq[jr],
 		  ChainID[jr],type, pa_int, syn_i, syn_j, corresp);
 
 	  char *s;
@@ -329,8 +293,8 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	  list->pair[num_bp].jc     = bseq[j];
 	  status = CMAP_String2BPTYPE(type, &list->pair[num_bp].bptype, errbuf);
 	  if (status != eslOK)  {
-	    printf("wrong BYTYPE %s\n", type);
-	    ESL_XFAIL(eslFAIL, errbuf, "wrong BYTYPE %s", type);
+	    printf("wrong BPTYPE %s\n", type);
+	    ESL_XFAIL(eslFAIL, errbuf, "wrong BPTYPE %s", type);
 	  }
 
 	  num_bp++;
@@ -350,20 +314,20 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	}
         
         
-	if (++pair_info[i][NP] >= NP) {
+	if (++pair_info[i][NPT] >= NPT) {
 	  printf( "residue %s has over %ld pairs\n", b1,
-		  NP - 1);
-	  --pair_info[i][NP];
+		  NPT - 1);
+	  --pair_info[i][NPT];
 	  break;
 	} else
-	  pair_info[i][pair_info[i][NP]] = j;
-	if (++pair_info[j][NP] >= NP) {
+	  pair_info[i][pair_info[i][NPT]] = j;
+	if (++pair_info[j][NPT] >= NPT) {
 	  printf( "residue %s has over %ld pairs\n", b2,
-		  NP - 1);
-	  --pair_info[j][NP];
+		  NPT - 1);
+	  --pair_info[j][NPT];
 	  break;
 	} else
-	  pair_info[j][pair_info[j][NP]] = i;
+	  pair_info[j][pair_info[j][NPT]] = i;
 	
       }
       else if( bpid_lu ==0){ /* not a H bond for base -  base */
@@ -494,7 +458,7 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
 	    
 	    if(nb_other++ > num_residue-2){
 	      printf("increase memory for single H bond (other cases)\n");
-	      fprintf(fout,"END_base-pair\n" );
+	      fprintf(fout, "END_base-pair\n" );
 	      return eslFAIL;
               
 	    }
@@ -541,7 +505,7 @@ int all_pairs(char *pdbfile, FILE *fout, long num_residue, long *RY,
   free_cmatrix(hb_atom1, 0, BUF512, 0, 4);
   free_cmatrix(hb_atom2, 0, BUF512, 0, 4);
   free_dvector(hb_dist, 0, BUF512);
-  free_lmatrix(pair_info, 0, num_residue, 0, NP);
+  free_lmatrix(pair_info, 0, num_residue, 0, NPT);
   free_lvector(prot_rna, 0, num_residue);
 
   er_ListDump(stdout, list);
@@ -1388,7 +1352,7 @@ void protein_rna_interact(double H_limit, long num_residue, long **seidx,
     long     i, ir, jr, j, k;
     long     n = 0;
     long     m;
-    long     interact, prot_atom = 0;
+    long     interact, prot_atom = 0, prot_atom_tot;
 
     /* read in the PDB file */
     pr_AtomName = cmatrix(1, prot_atom, 0, 4);
@@ -1396,7 +1360,7 @@ void protein_rna_interact(double H_limit, long num_residue, long **seidx,
     if ((finp = fopen("protein.pdb", "r")) == NULL) {
         printf("Can not open the file: protein.pdb (routine: protein_rna_interact)\n");
     }
-    prot_atom = number_of_atoms("protein.pdb");
+    prot_atom = number_of_atoms("protein.pdb", &prot_atom_tot);
 
     while(fgets(str, sizeof str, finp) != NULL) {
         strncpy(tmp, str + 12, 4);
