@@ -52,6 +52,7 @@ get_reference_pdb(char *BDIR)
   long     i,j,k,snum;
   double **sx;
   
+
   sAtomName = cmatrix(1, 20, 0, 4);
   sx = dmatrix(1, 20, 1, 3);
 
@@ -180,11 +181,12 @@ long upperstr(char *a)
 }
 
 
-long number_of_atoms(char *pdbfile)
+long number_of_atoms(char *pdbfile, long *ret_natom_tot)
 /* number of atom records in a PDB file */
 {
   char str[BUF512];
   long n = 0, nlen;
+  long natom_tot = 0;
   FILE *fp;
   
   if((fp = fopen(pdbfile, "r"))==NULL){
@@ -194,6 +196,8 @@ long number_of_atoms(char *pdbfile)
   
   while (fgets(str, sizeof str, fp) != NULL) {
     nlen = upperstr(str);
+    if (!strncmp(str, "ATOM", 4) || !strncmp(str, "HETATM", 6)) natom_tot ++;
+  
     if (!strncmp(str+17, "HOH", 3) || !strncmp(str+17, "WAT", 3))
       continue; /*get ride of WATER */  
     if (str[13] == 'H')
@@ -204,6 +208,8 @@ long number_of_atoms(char *pdbfile)
             n++;
   }
   fclose(fp);
+
+  *ret_natom_tot = natom_tot+1;
   return n;
 }
 
@@ -219,14 +225,15 @@ long read_pdb(char *pdbfile, char **AtomName, char **ResName, char *ChainID,
   char atomname[5], atomnum[7], resname[4], chainid, resseq[5];
   FILE *fp;
   
-  if((fp = fopen(pdbfile, "r"))==NULL) {        
+  if ((fp = fopen(pdbfile, "r")) == NULL) {        
     printf("Can not open the file %s (routine: read_pdb)\n",pdbfile);
     return 0;
   }
   
-  n=1;
+  n = 1;
   while (fgets(str, sizeof str, fp) != NULL) {
-    nlen = upperstr(str);
+    // nlen = upperstr(str); ER: why upperstr?? this is wrong
+    nlen = strlen(str);
     if (!strncmp(str+17, "HOH", 3) || !strncmp(str+17, "WAT", 3))
       continue; /*get ride of WATER */
     
@@ -234,13 +241,14 @@ long read_pdb(char *pdbfile, char **AtomName, char **ResName, char *ChainID,
       continue;  /*get ride of H atoms */
     
     if ((nlen >= 54 && !strncmp(str, "ATOM", 4))
-	|| (!strncmp(str, "HETATM", 6) && HETA>0) ){
+	|| (!strncmp(str, "HETATM", 6) && HETA>0) ) {
       
       strncpy(atomname, str + 12, 4);
       atomname[4] = '\0';
       
       strncpy(resname, str + 17, 3);        /* residue name */
       resname[3] = '\0';
+      
       /* delete ending spaces as in "C  " */
       for (i = 1; i <= 2; i++)
 	if (resname[2] == ' ') {
@@ -253,7 +261,7 @@ long read_pdb(char *pdbfile, char **AtomName, char **ResName, char *ChainID,
 	nrerror("==> residue name field empty <==");
       }
       
-      chainid=str[21];
+      chainid = str[21];
       
       strncpy(resseq, str + 22, 4);    /* residue sequence */
       resseq[4] = '\0';
@@ -274,20 +282,18 @@ long read_pdb(char *pdbfile, char **AtomName, char **ResName, char *ChainID,
       
       strncpy(atomnum, str + 6, 6);    /* atom number */
       atomnum[6] = '\0';
-     if (sscanf(atomnum, "%6ld", &AtomNum[n]) != 1) {
+      
+      if (sscanf(atomnum, "%6ld", &AtomNum[n]) != 1) {
 	printf( "atom #? ==> %.54s\n", str);
 	exit(1);
       }
-
-      
-      
+ 
       strncpy(temp, str + 30, 25);           /* xyz */
       temp[25] = '\0';
-      if (sscanf(temp,"%8lf%8lf%8lf",
-		 &xyz[n][1],&xyz[n][2],&xyz[n][3])!=3)
+      if (sscanf(temp,"%8lf%8lf%8lf", &xyz[n][1], &xyz[n][2], &xyz[n][3]) != 3)
 	nrerror("error reading xyz-coordinate");
       
-      Miscs[n][0] = str[0];        /* H for HETATM, A for ATOM */
+      Miscs[n][0] = str[0];         /* H for HETATM, A for ATOM */
       Miscs[n][1] = str[16];        /* alternative location indicator */
       Miscs[n][2] = str[26];        /* code of insertion residues */
       strncpy(Miscs[n] + 3, str + 54, NMISC - 3);
@@ -296,32 +302,31 @@ long read_pdb(char *pdbfile, char **AtomName, char **ResName, char *ChainID,
       else
 	Miscs[n][NMISC] = '\0';                /* just to make sure */
       
-      
       if (AtomName[n][3] == '*')        /* * to ' */
 	AtomName[n][3] = '\'';
-      if (!strcmp(AtomName[n], " O1'")){        /* O1' to O4' */
+      if        (!strcmp(AtomName[n], " O1'")) {       /* O1' to O4' */
 	strcpy(AtomName[n], " O4'");
-      }else if (!strcmp(AtomName[n], " OL ")) {       /* OL to O1P */
+      } else if (!strcmp(AtomName[n], " OL ")) {       /* OL to O1P */
 	strcpy(AtomName[n], " O1P");
-      }else if (!strcmp(AtomName[n], " OR ")) {       /* OR to O2P */
+      } else if (!strcmp(AtomName[n], " OR ")) {       /* OR to O2P */
 	strcpy(AtomName[n], " O2P");
-      }else if  (!strcmp(AtomName[n], " C5A")){        /* C5A to C5M */
+      } else if (!strcmp(AtomName[n], " C5A")) {       /* C5A to C5M */
 	strcpy(AtomName[n], " C5M");
-      }else if (!strcmp(AtomName[n], " O5T")){        /* terminal O5' */
+      } else if (!strcmp(AtomName[n], " O5T")) {       /* terminal O5' */
 	strcpy(AtomName[n], " O5'");
-      }else if (!strcmp(AtomName[n], " O3T")){        /* terminal O3' */
+      } else if (!strcmp(AtomName[n], " O3T")) {       /* terminal O3' */
 	strcpy(AtomName[n], " O3'");
       }
-      /*            
-		    printf("%s%5ld %4s%c%3s %c%4ld%c   %8.3lf%8.3lf%8.3lf\n", 
-		    "ATOM  ", n, AtomName[n], Miscs[n][1],
-		    ResName[n], ChainID[n], ResSeq[n], Miscs[n][2], xyz[n][1],
-		    xyz[n][2], xyz[n][3]);
-      */           
+          
+      if (0) printf("fpair_sub ^^num %s %ld AtomNum %ld AtomName%4s%c%3s |%c%4ld%c|   %8.3lf%8.3lf%8.3lf\n",
+		    "ATOM  ", n, AtomNum[n],
+		    AtomName[n], Miscs[n][1], ResName[n],
+		    ChainID[n], ResSeq[n], Miscs[n][2],
+		    xyz[n][1], xyz[n][2], xyz[n][3]);      
       
       n++;
       
-      sprintf(str_id0, "%s%s%c%s",atomname, resname, chainid, resseq); 
+      sprintf(str_id0, "%s%s%c%s", atomname, resname, chainid, resseq); 
       
     }
   }
@@ -514,38 +519,37 @@ long **residue_idx(long num, long *ResSeq, char **Miscs, char *ChainID,
                    char **ResName, long *num_residue)
 /* number of residues, and starting-ending indexes for each */
 {
-  char iCode;
+  char   iCode;
   char **bidx;
-  long i, n, **seidx, *temp;
-  
+  long **seidx;
+  long  *temp;
+  long   i;
+  long   n;
+
   bidx = cmatrix(1, num, 0, 12);        /* normally 9 */
   temp = lvector(1, num);
 
   for (i = 1; i <= num; i++) {
     iCode = (Miscs == NULL) ? ' ' : Miscs[i][2];
-    sprintf(bidx[i],"%3s%c%4ld%c",ResName[i],ChainID[i],ResSeq[i], iCode);
+    sprintf(bidx[i], "%3s%c%4ld%c", ResName[i], ChainID[i], ResSeq[i], iCode);
   }
-  for (i = 1; i < num; i++) {
+  
+  for (i = 1; i < num; i++)     
     temp[i] = strcmp(bidx[i + 1], bidx[i]) ? 1 : 0;
-  }
   temp[num] = 1;
   
+  
   n = 0;                        /* get number of residues */
   for (i = 1; i <= num; i++)
-    if (temp[i])
-      ++n;
+    if (temp[i]) ++n;
+  *num_residue = n;
   
   seidx = lmatrix(1, n, 1, 2);        /* allocate spaces */
   n = 0;
-  for (i = 1; i <= num; i++)
-    if (temp[i])
-      seidx[++n][2] = i;
-  for (i = 2; i <= n; i++)
-    seidx[i][1] = seidx[i - 1][2] + 1;
+  for (i = 1; i <= num;          i++) if (temp[i]) seidx[++n][2] = i;
+  for (n = 2; n <= *num_residue; n++) seidx[n][1] = seidx[n - 1][2] + 1;
   seidx[1][1] = 1;
 
-  *num_residue = n;
-  
   free_cmatrix(bidx, 1, num, 0, 12);
   free_lvector(temp, 1, num);
   
@@ -556,7 +560,7 @@ long **residue_idx(long num, long *ResSeq, char **Miscs, char *ChainID,
 long residue_ident(char **AtomName, double **xyz, long ib, long ie)
 /* identifying a residue as follows:
  *  R-base  Y-base  amino-acid, others [default]
- *   +1        0        -1        -2 [default]
+ *   +1        0        -1        -2   [default]
  */
 {
     double d1, d2, d3, dcrt = 2.0, dcrt2 = 3.0, temp[4];
@@ -569,15 +573,15 @@ long residue_ident(char **AtomName, double **xyz, long ib, long ie)
     C6 = find_1st_atom(" C6 ", AtomName, ib, ie, "");
 
     if (N1 && C2 && C6) {
-        for (i = 1; i <= 3; i++)
-            temp[i] = xyz[N1][i] - xyz[C2][i];
+        for (i = 1; i <= 3; i++) temp[i] = xyz[N1][i] - xyz[C2][i];
         d1 = veclen(temp);
-        for (i = 1; i <= 3; i++)
-            temp[i] = xyz[N1][i] - xyz[C6][i];
+	
+        for (i = 1; i <= 3; i++) temp[i] = xyz[N1][i] - xyz[C6][i];
         d2 = veclen(temp);
-        for (i = 1; i <= 3; i++)
-            temp[i] = xyz[C2][i] - xyz[C6][i];
+	
+        for (i = 1; i <= 3; i++) temp[i] = xyz[C2][i] - xyz[C6][i];
         d3 = veclen(temp);
+	
         if (d1 <= dcrt && d2 <= dcrt && d3 <= dcrt2) {
             id = 0;
             if (N9) {
@@ -591,16 +595,16 @@ long residue_ident(char **AtomName, double **xyz, long ib, long ie)
         return id;
     }
     CA = find_1st_atom(" CA ", AtomName, ib, ie, "");
-    C = find_1st_atom(" C  ", AtomName, ib, ie, "");
+    C  = find_1st_atom(" C  ", AtomName, ib, ie, "");
     if (!C)                        /* if C does not exist, use N */
         C = find_1st_atom(" N  ", AtomName, ib, ie, "");
     if (CA && C) {
-        for (i = 1; i <= 3; i++)
-            temp[i] = xyz[CA][i] - xyz[C][i];
+        for (i = 1; i <= 3; i++) temp[i] = xyz[CA][i] - xyz[C][i];
         if (veclen(temp) <= dcrt)
             id = -1;
         return id;
     }
+
     return id;                        /* other cases */
 }
 
@@ -2084,8 +2088,8 @@ void bp_network(long num_residue, long *RY, long **seidx, char **AtomName,
 	     bseq[i], 1, b1);
     
     /* list of direct pairing 
-       fprintf(fp, "%5ld %s: [%2ld]", i, b1, pair_info[i][NP]);
-       for (j = 1; j <= pair_info[i][NP]; j++)
+       fprintf(fp, "%5ld %s: [%2ld]", i, b1, pair_info[i][NPT]);
+       for (j = 1; j <= pair_info[i][NPT]; j++)
        fprintf(fp, "%5ld", pair_info[i][j]);
        fprintf(fp, "\n");*/
     
@@ -2098,7 +2102,7 @@ void bp_network(long num_residue, long *RY, long **seidx, char **AtomName,
     m = 1;
     while (ivec[m] && m <= tnum_base) {
       ir = ivec[m++];
-      for (j = 1; j <= pair_info[ir][NP]; j++) {
+      for (j = 1; j <= pair_info[ir][NPT]; j++) {
 	for (k = 1; k <= inum_base; k++)
 	  if (pair_info[ir][j] == ivec[k])
 	    break;
@@ -2144,23 +2148,23 @@ void bp_network(long num_residue, long *RY, long **seidx, char **AtomName,
     k = 0;
     for (j = 2; j <= inum_base; j++)
       if (ivec[j] > 0) {
-	if (++k >= NP) {
+	if (++k >= NPT) {
 	  printf( "residue %s has over %ld pairs\n", b1,
-		  NP - 1);
+		  NPT - 1);
 	  --k;
 	  break;
 	} else
 	  pair_info[i][k] = ivec[j];
       }
-    pair_info[i][NP] = k;        /* total number of pairs for residue i */
+    pair_info[i][NPT] = k;        /* total number of pairs for residue i */
     if (k++ > 1) {
       num_ple++;
       if (k > max_ple)
 	max_ple = k;
     }
     /*
-      fprintf(fp, "                      [%2ld]", pair_info[i][NP]);
-      for (j = 1; j <= pair_info[i][NP]; j++)
+      fprintf(fp, "                      [%2ld]", pair_info[i][NPT]);
+      for (j = 1; j <= pair_info[i][NPT]; j++)
       fprintf(fp, "%5ld", pair_info[i][j]);
       fprintf(fp, "\n");
     */
@@ -2199,10 +2203,10 @@ void multiplets(long num_ple, long max_ple, long num_residue,
   xyz_residue = dmatrix(1, NUM_RESIDUE_ATOMS, 1, 3);
   
   for (i = 1; i <= num_residue; i++) {
-    if (pair_info[i][NP] > 1) {
-      inum_base = pair_info[i][NP] + 1;
+    if (pair_info[i][NPT] > 1) {
+      inum_base = pair_info[i][NPT] + 1;
       ivec[1] = i;
-      for (j = 1; j <= pair_info[i][NP]; j++)
+      for (j = 1; j <= pair_info[i][NPT]; j++)
 	ivec[j + 1] = pair_info[i][j];
       lsort(inum_base, ivec, idx1);        /* sort into order */
       is_exist = 0;        /* check if already counted */
