@@ -24,7 +24,6 @@ static void delete_file(char *pdbfile, char *extension);
 
 int main(int argc, char *argv[])
 {
-  clock_t start, finish;
   long    i;
   
   ARGV=cmatrix(0, 6, 0, 40);/*6 argument and 40 length each */
@@ -34,8 +33,6 @@ int main(int argc, char *argv[])
     strcpy(ARGV[i], argv[i]);
   }
   
-  start = clock();
-  
   if (argc <= 1 || (argc == 2 && strstr(argv[1], "-h")))
     usage();
   if ( (strstr(argv[1], "a") || strstr(argv[1], "A")) && argv[1][0] == '-' ){
@@ -48,9 +45,6 @@ int main(int argc, char *argv[])
     process_single_file(argc, argv);
   }
   
-  finish = clock();
-  //printf( "\nTime used: %.2f seconds\n", ((double) (finish - start)) / CLOCKS_PER_SEC);
-  //fprintf(stderr, "\nJOB finished! Time used: %.2f seconds\n", ((double) (finish - start)) / CLOCKS_PER_SEC);
   free_cmatrix(ARGV, 0, 6, 0,40);
  
   return 0;    
@@ -105,17 +99,10 @@ void process_single_file(int argc, char *argv[])
       usage();
   }
   
-  
-  /*	strcpy(pdbfile,"P4-P6-A.pdb");	*/
-  
-  /*    check_cif(pdbfile, &yes);    
-	finp = fopen(pdbfile,"r");
-  */
+  // er: not sure what has to change when the data is NMR  
   check_nmr_xray(pdbfile, &key, outfile); /* key=0 nmr; key=1 xray */
-  if(key==0){
-    strcpy(pdbfile, outfile);
-  }
-  
+  if (key == 0) strcpy(pdbfile, outfile);
+
   rna(pdbfile, type_stat, pair_stat, &base_all);
   if(XML==0)
     base_edge_stat(pdbfile, A, U, G, C, T, P, I);
@@ -298,38 +285,36 @@ void rna_select(char *pdbfile, double resolution, long *yes)
 
 void check_nmr_xray(char *inpfile, long *key, char *outfile)
 {
-    long yes=0;
-    char str[256];
+    long  yes = 0;
+    char  str[256];
     FILE *fp;
     
     *key = -1;
     
-/*    del_extension(inpfile, parfile);*/ 
-   
-    sprintf(outfile, "%s_nmr.pdb", inpfile);
+     sprintf(outfile, "%s_nmr.pdb", inpfile);
     
-    if((fp=fopen(inpfile, "r"))==NULL) {        
+    if ((fp = fopen(inpfile, "r"))==NULL) {        
         printf("Can not open the INPUT file %s (routine:check_nmr_xray)\n", inpfile);
         return;
     }
     
     while (fgets(str, sizeof str, fp) != NULL) {
         upperstr(str);
-        if(!strncmp(str, "EXPDTA", 6) && strstr(str, "  X-RAY")){
+        if (!strncmp(str, "EXPDTA", 6) && strstr(str, "  X-RAY")){
             *key = 1;            
             fclose(fp);
             return;
-        }else if (!strncmp(str, "EXPDTA", 6) && strstr(str, "  NMR")){           
+        } else if (!strncmp(str, "EXPDTA", 6) && strstr(str, "  NMR")){           
             *key = 0;            
             extract_nmr_coord(inpfile,  outfile);
             fclose(fp);
             return;
         }
     }
-    if(*key == -1){  /* no EXPDTA term */
+    if (*key == -1){  /* no EXPDTA term */
         fp=fopen(inpfile, "r");
         check_model(fp, &yes);
-        if(yes==1) {    /* nmr structure */
+        if (yes==1) {    /* nmr structure */
             *key = 0;            
             extract_nmr_coord(inpfile,  outfile);
             fclose(fp);
@@ -341,7 +326,6 @@ void check_nmr_xray(char *inpfile, long *key, char *outfile)
         }
         
     }
-
 }
 
 void usage(void)
@@ -400,85 +384,101 @@ void contact_msg(void)
 
 void extract_nmr_coord(char *inpfile,  char *outfile)
 /* extract the nmr coordinate. If found the best, write the best to
-   parfile_nmr.pdb, if not , write the first model to parfile_nmr.pdb
+   parfile_nmr.pdb, if not, write the first model to parfile_nmr.pdb
 */
 {
-    long j,  model;
-    char str[256],substr[80];
-    FILE *fp, *fout;
+  FILE *fp   = NULL;
+  FILE *fout = NULL;
+  char  str[256];
+  char  substr[80];
+  long  model;
+  long  j;
+  
+  fout = fopen(outfile, "w");
+  
+  if ((fp = fopen(inpfile, "r")) == NULL) {        
+    printf("Can not open the INPUT file %s (routine:extract_nmr_coord)\n", inpfile);
+    exit(0);
+  }
+  
+  model = -999;
+  while (fgets(str, sizeof str, fp) != NULL) {
     
-    fout = fopen(outfile, "w");
+    upperstr(str);
     
-    if((fp=fopen(inpfile, "r"))==NULL) {        
-        printf("Can not open the INPUT file %s(routine:extract_nmr_coord)\n", inpfile);
-        exit(0);
+    if (!strncmp(str, "REMARK", 6) && strstr(str, "BEST REPRESENTATIVE CONFORMER IN THIS ENSEMBLE :") ) {
+      strcpy(substr, strstr(str,":")+1);
+      
+      if (sscanf(substr, "%ld", &j) == 1) model = j;
+      else                                model = -999;
     }
-
-    model = -999;
+    if (!strncmp(str, "SEQRES", 6) || !strncmp(str, "ATOM", 4)  || !strncmp(str, "HETA", 4) )
+      break;
+  }
+  
+  if (model == -999)
+    printf("The best NMR model is not available. The first model is used.!\n");
+  else
+    printf("The best representative of NMR model = %ld\n", model);
+  
+  rewind(fp);
+  if (model == -999) {
+    fprintf(fout, "REMARK   NMR structure:    model = 1\n");
+    
     while (fgets(str, sizeof str, fp) != NULL) {
-        upperstr(str);
-        if(!strncmp(str, "REMARK", 6) &&
-           strstr(str, "BEST REPRESENTATIVE CONFORMER IN THIS ENSEMBLE :") ){
-            strcpy(substr, strstr(str,":")+1);
-            if(sscanf(substr, "%ld", &j)==1) {
-                model=j;
-            }else
-                model = -999;
-        }
-        if(!strncmp(str, "ATOM", 4)  || !strncmp(str, "HETA", 4) )
-            break;
+      upperstr(str);
+      
+      if (!strncmp(str, "SEQRES", 6) || !strncmp(str, "ATOM", 4)  || !strncmp(str, "HETA", 4) ) {
+	fprintf(fout, "%s", str);
+	
+	while (fgets(str, sizeof str, fp) != NULL) {
+	  if (!strncmp(str, "MODEL",  5) ||
+	      !strncmp(str, "ENDMDL", 6) ||
+	      !strncmp(str, "END",    3)   ) {
+	    fprintf(fout, "END\n");
+	    fclose(fout);
+	    return;
+	  }
+	  fprintf(fout,"%s", str);
+	}
+      }
     }
-    if(model == -999)
-        printf("The best NMR model is not available. The first model is used.!\n");
-    else
-        printf("The best representative of NMR model = %ld\n", model);
+  }
+  else {
+    
+    j=-999;
+    
+    while (fgets(str, sizeof str, fp)) {
+      // er: need to keep the seq as well
+      if (!strncmp(str, "SEQRES", 6)) fprintf(fout, "%s", str);
 
-    rewind(fp);
-    if(model == -999){
-        fprintf(fout,"REMARK   NMR structrue:    model = 1\n"); 
-        while (fgets(str, sizeof str, fp) != NULL) {
-            upperstr(str);
-            if(!strncmp(str, "ATOM", 4)  || !strncmp(str, "HETA", 4) ){
-                fprintf(fout,"%s", str);
-                while (fgets(str, sizeof str, fp) != NULL) {
-                    if(!strncmp(str, "MODEL", 5) || !strncmp(str, "ENDMDL", 6)
-                       || !strncmp(str, "END", 3)){
-                        fprintf(fout,"END\n");
-                        fclose(fout);
-                        return;
-                    }
-                    fprintf(fout,"%s", str);
-                }
-            }
-        }
-    }else{
-        j=-999;
-        while (fgets(str, sizeof str, fp) != NULL) {
-            if(!strncmp(str, "MODEL", 5) && sscanf(str, "%*s %ld", &j)==1){
-                if(j == model) break;
-            }
-        }
-        if(j != -999){ /*found model*/
-            fprintf(fout,"REMARK  NMR structrue: (the best model = %ld ).\n",j); 
-                        
-            while (fgets(str, sizeof str, fp) != NULL) {
-                if(!strncmp(str, "MODEL", 5) || !strncmp(str, "ENDMDL", 6)
-                   || !strncmp(str, "END", 3)){
-                    fprintf(fout,"END\n");
-                    fclose(fout);
-                    return;
-                }
-                fprintf(fout,"%s", str);
-            }
-        }else{ /* not found model*/
-            rewind(fp);
-            while (fgets(str, sizeof str, fp) != NULL) {
-                fprintf(fout,"%s", str);
-            }
-        }
-                
+	if (!strncmp(str, "MODEL", 5) && sscanf(str, "%*s %ld", &j) == 1) {
+	if (j == model) break;
+      }
     }
     
+    if (j != -999) { // model found
+      fprintf(fout,"REMARK  NMR structrue: (the best model = %ld ).\n", j); 
+      
+      while (fgets(str, sizeof str, fp)) {
+	if (!strncmp(str, "MODEL",  5) ||
+	    !strncmp(str, "ENDMDL", 6) ||
+	    !strncmp(str, "END",    3)   ) {
+	  fprintf(fout,"END\n");
+	  fclose(fout);
+	  return;
+	}
+	fprintf(fout, "%s", str);
+      }
+    }
+    else {  // model not found
+      rewind(fp);
+      
+      while (fgets(str, sizeof str, fp) != NULL) {
+	fprintf(fout,"%s", str);
+      }
+    } 
+  }
 }
     
 
@@ -565,11 +565,12 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
   char HB_ATOM[BUF512], ALT_LIST[BUF512];
   char *user_chain = NULL;
   char *ChainID, *bseq, **AtomName, **ResName, **Miscs;
-  long i, j, k, m, n, ie, ib, dna_rna, num, num_residue, nres, bs_atoms;
+  long i, j, k, m, ie, ib, dna_rna, num, num_residue, nres, bs_atoms;
   long *ResSeq, *RY, **seidx, num_modify, *modify_idx;
   long *AtomNum, *Atom2SEQ;
   long nchain_tot;
   long **chain_idx, nchain;
+  long natom_tot;
   double HB_UPPER[2], **xyz;
   static long base_all;
   char  *chain_name = NULL;
@@ -586,84 +587,87 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
   
   sprintf(outfile, "%s.out", pdbfile);
   
-  fout=fopen(outfile, "w");
-  /*    prot_out = fopen("protein.pdb", "w");*/
+  fout = fopen(outfile, "w");
   
   /* read in H-bond length upper limit etc from <misc_rna.par> */    
   hb_crt_alt(HB_UPPER, HB_ATOM, ALT_LIST);
   
   /* read in the PDB file */
-  num = number_of_atoms(pdbfile);
+  num      = number_of_atoms(pdbfile, &natom_tot);
+  
   AtomName = cmatrix(1, num, 0, 4);
-  ResName = cmatrix(1, num, 0, 3);
-  ChainID = cvector(1, num);
-  AtomNum = lvector(1, num);
-  Atom2SEQ = lvector(1, num);
-  ResSeq = lvector(1, num);
-  xyz = dmatrix(1, num, 1, 3);
-  Miscs = cmatrix(1, num, 0, NMISC);
+  ResName  = cmatrix(1, num, 0, 3);
+  ChainID  = cvector(1, num);
+  AtomNum  = lvector(1, num);
+  Atom2SEQ = lvector(1, natom_tot);
+  ResSeq   = lvector(1, num);
+  xyz      = dmatrix(1, num, 1, 3);
+  Miscs    = cmatrix(1, num, 0, NMISC);
   
-  fprintf(stdout, "# PDB_file %s\n",  pdbfile);
-  fprintf(fout,"PDB data file name: %s\n",  pdbfile);
+  fprintf(stdout, "# PDB_file %s\n",           pdbfile);
+  fprintf(fout,   "PDB data file name: %s\n",  pdbfile);
   num = read_pdb(pdbfile, AtomName, ResName, ChainID, AtomNum, ResSeq, xyz, Miscs, ALT_LIST);
-  
+  if (num == 0) { printf("pdfile %s has no residues (case #1)\n", pdbfile); exit(1); }
+
   /* get the numbering information of each residue.
      seidx[i][j]; i = 1-num_residue  j=1,2
   */
   seidx = residue_idx(num, ResSeq, Miscs, ChainID, ResName, &num_residue);
-  
+
   if (CHAIN == 1){
     esl_sprintf(&user_chain, ARGV[2]);
-    upperstr(user_chain);
+    //upperstr(user_chain); ER: wrong to assume that names of chains use all capital letters. They don't have to, exammple: 8cgv has chaisn a,b,c,d...
   }
-  
+
   if (er_PrintChainSeqs(pdbfile, user_chain, ChainID, num_residue, seidx, ResName,
 			AtomNum, Atom2SEQ, ResSeq, AtomName, Miscs, xyz,
 			&nchain_tot, &chain_name, &chain_f, &chain_t, errbuf) != eslOK) {
     printf("%s\n", errbuf);
     exit(1);
   }
-  
-  /* Below is only for nucleic acids ie RY >= 0*/  
+   
+  /* Below is only for nucleic acids ie RY >= 0 */  
   bs_atoms = 0;
-   for (i = 1; i <= num_residue; i++){
+  for (i = 1; i <= num_residue; i++){
     ib = seidx[i][1];
     ie = seidx[i][2];
     dna_rna = residue_ident(AtomName, xyz, ib, ie);
     
-    //if (dna_rna < 0) printf("   removing chain %c residue %ld %s (%ld-%ld atoms)\n", ChainID[ib], i, ResName[ib], ib, ie);
-    
     if (dna_rna >= 0){         
       
       for (j = seidx[i][1]; j <= seidx[i][2]; j++){
 	if (CHAIN == 0) { 
 	  bs_atoms++;
+	  
 	  if (bs_atoms != j) { // ER: copying a string to itself causes problems in some OS
 	    strcpy(AtomName[bs_atoms], AtomName[j]);
-	    strcpy(ResName[bs_atoms], ResName[j]);
+	    strcpy(ResName[bs_atoms],  ResName[j]);
+	    
 	    ChainID[bs_atoms] = ChainID[j];
-	    ResSeq[bs_atoms] = ResSeq[j];
+	    ResSeq[bs_atoms]  = ResSeq[j];
 	    AtomNum[bs_atoms] = AtomNum[j];
+	  
+	    for(k = 0 ; k <= NMISC; k++) Miscs[bs_atoms][k] = Miscs[j][k];
+	    for(k = 1 ; k <= 3;     k++) xyz[bs_atoms][k]   = xyz[j][k];
 	  }
-	  for(k = 0 ; k <=NMISC; k++)
-	    Miscs[bs_atoms][k] = Miscs[j][k];
-	  for(k = 1 ; k <=3; k++)
-	    xyz[bs_atoms][k] = xyz[j][k];
 	}
-	else { /* user select chainiD */
-	  if(!strchr(user_chain, toupper(ChainID[j]) ) )continue;
+	else { /* user select chainID */
+	  // if (!strchr(user_chain, toupper(ChainID[j]) )) continue; ER: wrong, cannot assume chain names use only capital letters
+	  if (!strchr(user_chain, ChainID[j] )) continue;
+	  
 	  bs_atoms++;
+	  
 	  if (bs_atoms != j) {  // ER: copying a string to itself causes problems in some OS
 	    strcpy(AtomName[bs_atoms], AtomName[j]);
-	    strcpy(ResName[bs_atoms], ResName[j]);
+	    strcpy(ResName[bs_atoms],  ResName[j]);
+	    
 	    ChainID[bs_atoms] = ChainID[j];
-	    ResSeq[bs_atoms] = ResSeq[j];
+	    ResSeq[bs_atoms]  = ResSeq[j];
 	    AtomNum[bs_atoms] = AtomNum[j];
+	  
+	    for(k = 0 ; k <= NMISC; k++) Miscs[bs_atoms][k] = Miscs[j][k];
+	    for(k = 1 ; k <= 3;     k++) xyz[bs_atoms][k]   = xyz[j][k];
 	  }
-	  for(k = 0 ; k <=NMISC; k++)
-	    Miscs[bs_atoms][k] = Miscs[j][k];
-	  for(k = 1 ; k <=3; k++)
-	    xyz[bs_atoms][k] = xyz[j][k];
 	}
       }
     }
@@ -675,9 +679,8 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
     */        
   }
   /*    fclose(prot_out);*/
-  
-  
-  /* get the new  numbering information of each residue */
+
+  /* get the new numbering information of each residue */
   
   /* get base sequence, RY identification */
   /* identifying a residue as follows:  RY[j]
@@ -686,18 +689,19 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
    
    bseq[j] --> the sigle letter name for each residue. j = 1 - num_residue.
   */
+  if (bs_atoms == 0) { printf("pdfile %s has no residues (case #2)\n", pdbfile); exit(1); }
   
   bseq = cvector(1, num_residue);
-  nres=0;    
-  seidx=residue_idx(bs_atoms, ResSeq, Miscs, ChainID, ResName, &nres);
+  nres = 0;
+  seidx = residue_idx(bs_atoms, ResSeq, Miscs, ChainID, ResName, &nres);
 
   nchain = nchain_tot;
-  chain_idx = lmatrix(1,500 , 1, 2);  /* # of chains max = 200 */    
+  chain_idx = lmatrix(1, 500 ,1, 2);  /* # of chains max = 200 */    
   get_chain_idx(nres, seidx, ChainID, &nchain, chain_idx);
 
   bs_atoms = 0;
   for (i=1; i<=nchain; i++){ /* rid of ligand */
-    if((chain_idx[i][2] - chain_idx[i][1]) <= 0)continue;
+    if ((chain_idx[i][2] - chain_idx[i][1]) <= 0)continue;
 
     for (k=chain_idx[i][1]; k<=chain_idx[i][2]; k++){
       ib = seidx[k][1];
@@ -718,21 +722,21 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
 	    Miscs[bs_atoms][m] = Miscs[j][m];
 	  for(m = 1 ; m <=3; m++)
 	    xyz[bs_atoms][m] = xyz[j][m];
-	  n=bs_atoms;
 	}
       }
 
     }
   }
-  nres=0;    
-  seidx=residue_idx(bs_atoms, ResSeq, Miscs, ChainID, ResName, &nres);
+  if (bs_atoms == 0) { printf("pdfile %s has no residues (case #3)\n", pdbfile); exit(1); }
+  
+  nres = 0;    
+  seidx = residue_idx(bs_atoms, ResSeq, Miscs, ChainID, ResName, &nres);
   
   RY = lvector(1, num_residue);
   modify_idx = lvector(1, num_residue);    
-  get_seq(fout,nres, seidx, AtomName, ResName, ChainID, ResSeq, Miscs,
-	  xyz, bseq, RY, &num_modify,modify_idx);  /* get the new RY */
+  get_seq(fout, nres, seidx, AtomName, ResName, ChainID, ResSeq, Miscs,
+	  xyz, bseq, RY, &num_modify, modify_idx);  /* get the new RY */
   
- 
   work_horse(pdbfile, fout, nres, bs_atoms, bseq, seidx, RY, AtomName,
 	     ResName, ChainID, nchain, chain_idx, AtomNum, Atom2SEQ, ResSeq,
 	     Miscs, xyz, nchain_tot, chain_name, chain_f, chain_t,
@@ -741,7 +745,7 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
   base_all=base_all+nres; /* accumulate all the bases */
   *bs_all=base_all;
   
-  if(!(PS>0 || VRML>0 || XML>0 || ALL>0) )
+  if (!(PS>0 || VRML>0 || XML>0 || ALL>0) )
     write_tmp_pdb(pdbfile,nres,seidx,AtomName,ResName,ChainID,ResSeq,xyz);
   
   
@@ -763,6 +767,7 @@ void rna(char *pdbfile, long *type_stat, long **pair_stat, long *bs_all)
   if (chain_t) free (chain_t);
 }
 
+/* perform all the calculations */
 void work_horse(char *pdbfile, FILE *fout, long num_residue, long num,
                 char *bseq, long **seidx, long *RY, char **AtomName,
                 char **ResName, char *ChainID, long nchain, long **chain_idx,
@@ -771,27 +776,27 @@ void work_horse(char *pdbfile, FILE *fout, long num_residue, long num,
 		long nchain_tot, char *chain_name, long *chain_f, long *chain_t,
 		long num_modify, long *modify_idx, 
                 long *type_stat,long **pair_stat)
-/* perform all the calculations */
-  
 {    
-  long **bs_pairs_tot, num_pair_tot=0, num_single_base=0,*single_base, ntot;
+  long **bs_pairs_tot, num_pair_tot=0, num_single_base=0,*single_base, ntot=0;
   long *sugar_syn;
   double BPRS[7];
   char **pair_type;
   double **orien, **org, **Nxyz, **o3_p;
   long num_multi, *multi_idx, **multi_pair;
+
+  if (num_residue == 0) { printf("No residues\n"); exit(1); }
   
-  multi_pair = lmatrix(1, num_residue, 1, 20); /* max 20-poles */
-  multi_idx = lvector(1, num_residue);  /*max multipoles = num_residue */
-  pair_type = cmatrix(1, num_residue*2, 0, 3); /* max base pairs */    
+  multi_pair   = lmatrix(1, num_residue, 1, 20);   /* max 20-poles */
+  multi_idx    = lvector(1, num_residue);          /* max multipoles = num_residue */
+  pair_type    = cmatrix(1, num_residue*2, 0, 3);  /* max base pairs */    
   bs_pairs_tot = lmatrix(1, 2*num_residue, 1, 2);
-  single_base = lvector(1, num_residue);  /* max single base */   
-  sugar_syn = lvector(1, num_residue); 
+  single_base  = lvector(1, num_residue);          /* max single base */   
+  sugar_syn    = lvector(1, num_residue); 
   
   orien = dmatrix(1, num_residue, 1, 9);
-  org = dmatrix(1, num_residue, 1, 3);
-  Nxyz = dmatrix(1, num_residue, 1, 3);     /* RN9/YN1 atomic coordinates */
-  o3_p = dmatrix(1, num_residue, 1, 8);     /* O3'/P atomic coordinates */
+  org   = dmatrix(1, num_residue, 1, 3);
+  Nxyz  = dmatrix(1, num_residue, 1, 3);     /* RN9/YN1 atomic coordinates */
+  o3_p  = dmatrix(1, num_residue, 1, 8);     /* O3'/P atomic coordinates */
   
   /* get the  base information for locating possible pairs later
      orien --> rotation matrix for each library base to match each residue.
@@ -801,7 +806,6 @@ void work_horse(char *pdbfile, FILE *fout, long num_residue, long num,
 	    ResSeq, Miscs, xyz,  orien, org, Nxyz, o3_p, BPRS);    
   
   /* find all the base-pairs */
-  //printf("Finding all the base pairs...\n");    
   all_pairs(pdbfile, fout, num_residue, RY, Nxyz, orien, org, BPRS,
 	    seidx, xyz, nchain_tot, chain_name, chain_f, chain_t,
 	    AtomName, ResName, ChainID, nchain, chain_idx,
@@ -1015,7 +1019,7 @@ void print_sorted_pair(long ntot, char *pdbfile)
   str_tmp = cmatrix(0, ntot, 0, 120);   
   index=lmatrix(0,25, 0, ntot); 
   
-  sprintf(inpfile, "%s.out", pdbfile);
+  sprintf(inpfile, "%s.out",      pdbfile);
   sprintf(outfile, "%s_sort.out", pdbfile);
   
   fout = fopen(outfile, "w");
