@@ -32,7 +32,7 @@
  *   8. Test driver
  *   9. Examples
  */
-#include "esl_config.h"
+#include <esl_config.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -75,17 +75,23 @@ static uint32_t eslDSQDATA_MAGIC_V1SWAP = 0xb1d1d3c4; //  ... as above, but byte
  * Synopsis:  Open a digital sequence database for reading
  * Incept:    SRE, Wed Jan 20 09:50:00 2016 [Amtrak 2150, NYP-BOS]
  *
- * Purpose:   Open digital sequence database <basename> for reading.
- *            Configure it for a specified number of 1 or
- *            more parallelized <nconsumers>. The consumers are one or
- *            more threads that are processing chunks of data in
- *            parallel.
+ * Purpose:   Open dsqdata database <basename> for reading.  The file
+ *            <basename> is a stub describing the database. The bulk
+ *            of the data are in three accompanying binary files: the
+ *            index file <basename>.dsqi, the metadata file
+ *            <basename>.dsqm, and the sequence file <basename>.dsqs.
  *            
- *            The file <basename> is a human-readable stub describing
- *            the database. The bulk of the data are in three
- *            accompanying binary files: the index file
- *            <basename>.dsqi, the metadata file <basename>.dsqm, and
- *            the sequence file <basename>.dsqs.
+ *            <nconsumers> is an upper bound on the number of threads
+ *            in which the caller plans to be calling
+ *            <esl_dsqdata_Read()> -- or, more precisely, the maximum
+ *            number of data chunks that the caller could be working
+ *            on at any given instant.  This is a hint, not a
+ *            commitment. The dsqdata loader uses it to determine the
+ *            maximum number of data chunks that can be in play at
+ *            once (including chunks it is juggling internally, plus
+ *            if all the caller's reader threads are busy on
+ *            theirs). If <nconsumers> is set too small, the loader
+ *            may stall waiting for chunks to come back for recycling.
  *            
  *            Reading digital sequence data requires a digital
  *            alphabet.  You can either provide one (in which case we
@@ -98,25 +104,25 @@ static uint32_t eslDSQDATA_MAGIC_V1SWAP = 0xb1d1d3c4; //  ... as above, but byte
  *            <*byp_abc> is a ptr to an existing alphabet, we use it
  *            for validation. That is, you have two choices:
  *                
- *            \begin{cchunk}
+ *            ```
  *                ESL_ALPHABET *abc = NULL;
  *                esl_dsqdata_Open(&abc, basename...)
  *                // <abc> is now the alphabet of <basename>; 
  *                // now you're responsible for Destroy'ing it
- *            \end{cchunk}
+ *            ```
  *                
  *            or:
  *
- *            \begin{cchunk}
+ *            ```
  *                ESL_ALPHABET *abc = esl_alphabet_Create(eslAMINO);
  *                status = esl_dsqdata_Open(&abc, basename);
  *                // if status == eslEINCOMPAT, alphabet in basename 
  *                // doesn't match caller's expectation
- *            \end{cchunk}
+ *            ```
  *
  * Args:      byp_abc    : expected or created alphabet; pass &abc, abc=NULL or abc=expected alphabet
  *            basename   : data are in files <basename> and <basename.dsq[ism]>
- *            nconsumers : number of consumer threads caller is going to Read() with
+ *            nconsumers : upper bound on number of consumer threads caller is going to Read() with
  *            ret_dd     : RETURN : the new ESL_DSQDATA object.
  *
  * Returns:   <eslOK> on success.
@@ -154,6 +160,7 @@ esl_dsqdata_Open(ESL_ALPHABET **byp_abc, char *basename, int nconsumers, ESL_DSQ
   uint32_t     magic     = 0;
   uint32_t     tag       = 0;
   uint32_t     alphatype = eslUNKNOWN;
+  int          ndd;
   char        *p;                       // used for strtok() parsing of fields on a line
   char         buf[4096];
   int          u;
@@ -190,15 +197,16 @@ esl_dsqdata_Open(ESL_ALPHABET **byp_abc, char *basename, int nconsumers, ESL_DSQ
 
   /* Open the four files.
    */
-  ESL_ALLOC( dd->basename, sizeof(char) * (strlen(basename) + 6)); // +5 for .dsqx; +1 for \0
-  if ( sprintf(dd->basename, "%s.dsqi", basename) <= 0)   ESL_XEXCEPTION_SYS(eslESYS, "sprintf() failure");
-  if (( dd->ifp = fopen(dd->basename, "rb"))   == NULL)   ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open index file %s\n", dd->basename);
+  ndd = strlen(basename) + 6;  // +5 for .dsqx; +1 for \0
+  ESL_ALLOC( dd->basename, ndd);
+  if ( snprintf(dd->basename, ndd, "%s.dsqi", basename) <= 0)   ESL_XEXCEPTION_SYS(eslESYS, "sprintf() failure");
+  if (( dd->ifp = fopen(dd->basename, "rb")) == NULL)           ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open index file %s\n", dd->basename);
 
-  if ( sprintf(dd->basename, "%s.dsqm", basename) <= 0)   ESL_XEXCEPTION_SYS(eslESYS, "sprintf() failure");
-  if (( dd->mfp = fopen(dd->basename, "rb"))   == NULL)   ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open metadata file %s\n", dd->basename);
+  if ( snprintf(dd->basename, ndd, "%s.dsqm", basename) <= 0)   ESL_XEXCEPTION_SYS(eslESYS, "sprintf() failure");
+  if (( dd->mfp = fopen(dd->basename, "rb")) == NULL)           ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open metadata file %s\n", dd->basename);
 
-  if ( sprintf(dd->basename, "%s.dsqs", basename) <= 0)   ESL_XEXCEPTION_SYS(eslESYS, "sprintf() failure");
-  if (( dd->sfp = fopen(dd->basename, "rb"))   == NULL)   ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open sequence file %s\n", dd->basename);
+  if ( snprintf(dd->basename, ndd, "%s.dsqs", basename) <= 0)   ESL_XEXCEPTION_SYS(eslESYS, "sprintf() failure");
+  if (( dd->sfp = fopen(dd->basename, "rb")) == NULL)           ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open sequence file %s\n", dd->basename);
 
   strcpy(dd->basename, basename);
   if (( dd->stubfp = fopen(dd->basename, "r")) == NULL)   ESL_XFAIL(eslENOTFOUND, dd->errbuf, "Failed to find or open stub file %s\n", dd->basename);
@@ -330,15 +338,23 @@ esl_dsqdata_Open(ESL_ALPHABET **byp_abc, char *basename, int nconsumers, ESL_DSQ
     }
   else if (status != eslESYS)
     {   /* on most exceptions, we free <dd>, return it NULL, don't change *byp_abc */
+      if (*byp_abc == NULL && dd->abc_r) esl_alphabet_Destroy(dd->abc_r);
       esl_dsqdata_Close(dd);
       *ret_dd = NULL;
-      if (*byp_abc == NULL && dd->abc_r) esl_alphabet_Destroy(dd->abc_r);
       return status;
     }
   else
-    { /* on eslESYS exceptions - pthread initializations failing - we can't assume we can _Close() correctly. */
-      *ret_dd = NULL;
+    { /* on eslESYS exceptions - pthread initializations failing - we can't assume we can _Close() correctly,
+       * and we have to do the cleanup verbosely
+       */
+      free(dd->basename);
+      if (dd->stubfp) fclose(dd->stubfp);
+      if (dd->ifp)    fclose(dd->ifp);
+      if (dd->sfp)    fclose(dd->sfp);
+      if (dd->mfp)    fclose(dd->mfp);
       if (*byp_abc == NULL && dd->abc_r) esl_alphabet_Destroy(dd->abc_r);
+      free(dd);
+      *ret_dd = NULL;
       return status;
     }
 }
@@ -588,6 +604,7 @@ esl_dsqdata_Write(ESL_SQFILE *sqfp, char *basename, char *errbuf)
   int64_t         spos        = 0;
   int64_t         mpos        = 0;
   int             n;
+  int             na;
   int             status;
 
   if (! esl_sqfile_IsRewindable(sqfp))  ESL_EXCEPTION(eslEINVAL, "sqfp must be rewindable (e.g. an open file)");
@@ -612,7 +629,7 @@ esl_dsqdata_Write(ESL_SQFILE *sqfp, char *basename, char *errbuf)
       esl_sq_Reuse(sq);
     }
   if      (status == eslEFORMAT) ESL_XFAIL(eslEFORMAT, errbuf, sqfp->get_error(sqfp));
-  else if (status != eslEOF)     return status;
+  else if (status != eslEOF)     goto ERROR;
 
   if ((status = esl_sqfile_Position(sqfp, 0)) != eslOK) return status;
 
@@ -624,17 +641,15 @@ esl_dsqdata_Write(ESL_SQFILE *sqfp, char *basename, char *errbuf)
   if      (alphatype == eslAMINO)                      do_pack5 = TRUE;
   else if (alphatype != eslDNA && alphatype != eslRNA) ESL_EXCEPTION(eslEINVAL, "alphabet must be protein or nucleic");
 
-
-  if (( status = esl_sprintf(&outfile, "%s.dsqi", basename)) != eslOK) goto ERROR;
-  if ((    ifp = fopen(outfile, "wb"))             == NULL)  ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata index file %s for writing", outfile);
-  sprintf(outfile, "%s.dsqm", basename);
-  if ((    mfp = fopen(outfile, "wb"))             == NULL)  ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata metadata file %s for writing", outfile);
-  sprintf(outfile, "%s.dsqs", basename);
-  if ((    sfp = fopen(outfile, "wb"))             == NULL)  ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata sequence file %s for writing", outfile);
-  if (( stubfp = fopen(basename, "w"))             == NULL)  ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata stub file %s for writing", basename);
-
-
-  
+  na = strlen(basename) + 6;  // '.dsq?' + '\0'
+  ESL_ALLOC(outfile, na);
+  if ( snprintf(outfile, na, "%s.dsqi", basename) < 0) ESL_XEXCEPTION(eslESYS,      "sprintf failed");
+  if (( ifp    = fopen(outfile, "wb")) == NULL)        ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata index file %s for writing", outfile);
+  if ( snprintf(outfile, na, "%s.dsqm", basename) < 0) ESL_XEXCEPTION(eslESYS,      "sprintf failed");
+  if (( mfp    = fopen(outfile, "wb")) == NULL)        ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata metadata file %s for writing", outfile);
+  if ( snprintf(outfile, na, "%s.dsqs", basename) < 0) ESL_XEXCEPTION(eslESYS,      "sprintf failed"); 
+  if (( sfp    = fopen(outfile, "wb")) == NULL)        ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata sequence file %s for writing", outfile);
+  if (( stubfp = fopen(basename, "w")) == NULL)        ESL_XFAIL(eslEWRITE, errbuf, "failed to open dsqdata stub file %s for writing", basename);
 
   /* Header: index file */
   if (fwrite(&magic,       sizeof(uint32_t), 1, ifp) != 1 ||
@@ -699,6 +714,8 @@ esl_dsqdata_Write(ESL_SQFILE *sqfp, char *basename, char *errbuf)
 
       esl_sq_Reuse(sq);
     }
+  if      (status == eslEFORMAT) ESL_XFAIL(eslEFORMAT, errbuf, sqfp->get_error(sqfp));
+  else if (status != eslEOF)     goto ERROR;
 
   /* Stub file */
   fprintf(stubfp, "Easel dsqdata v1 x%" PRIu32 "\n", uniquetag);
@@ -719,9 +736,9 @@ esl_dsqdata_Write(ESL_SQFILE *sqfp, char *basename, char *errbuf)
   return eslOK;
 
  ERROR:
-  if (sq)      esl_sq_Destroy(sq);
-  if (rng)     esl_randomness_Destroy(rng);
-  if (outfile) free(outfile);
+  esl_sq_Destroy(sq);
+  esl_randomness_Destroy(rng);
+  free(outfile);
   if (stubfp)  fclose(stubfp);
   if (ifp)     fclose(ifp);
   if (mfp)     fclose(mfp);
@@ -1042,7 +1059,7 @@ dsqdata_loader_thread(void *p)
    * sort to get the other threads to clean up and terminate.
    */
   if (idx) free(idx);    
-  esl_fatal("  ... dsqdata loader thread failed: unrecoverable");
+  esl_fatal("  ... dsqdata loader thread failed: unrecoverable error %d", status);
 }
 
 
@@ -1119,7 +1136,7 @@ dsqdata_unpacker_thread(void *p)
    * to tell other threads to clean up and terminate, we violate Easel standards
    * and turn nonfatal exceptions into fatal ones.
    */
-  esl_fatal("  ... dsqdata unpacker thread failed: unrecoverable"); 
+  esl_fatal("  ... dsqdata unpacker thread failed: unrecoverable error code %d", status); 
 }
 
 
@@ -1725,12 +1742,13 @@ main(int argc, char **argv)
 #include "esl_alphabet.h"
 #include "esl_dsqdata.h"
 #include "esl_getopts.h"
+#include "esl_vectorops.h"
 
 static ESL_OPTIONS options[] = {
   /* name             type          default  env  range toggles reqs incomp  help                                       docgroup*/
   { "-h",          eslARG_NONE,       FALSE,  NULL, NULL,  NULL,  NULL, NULL, "show brief help on version and usage",        0 },
-  { "-n",          eslARG_NONE,       FALSE,  NULL, NULL,  NULL,  NULL, NULL, "no residue counting: faster time version",    0 },
-  { "-t",          eslARG_INT,          "4",  NULL, "n>0", NULL,  NULL, NULL, "set number of threads to <n>",                0 },
+  { "-c",          eslARG_NONE,       FALSE,  NULL, NULL,  NULL,  NULL, NULL, "report summary of chunk contents",            0 },
+  { "-r",          eslARG_NONE,       FALSE,  NULL, NULL,  NULL,  NULL, NULL, "report summary of residue counts",            0 },
   {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
 };
 
@@ -1740,13 +1758,15 @@ static char banner[] = "example of using ESL_DSQDATA to read sequence db";
 int
 main(int argc, char **argv)
 {
-  ESL_GETOPTS       *go       = esl_getopts_CreateDefaultApp(options, 1, argc, argv, banner, usage);
-  ESL_ALPHABET      *abc      = NULL;
-  char              *basename = esl_opt_GetArg(go, 1);
-  int                no_count = esl_opt_GetBoolean(go, "-n");
-  int                ncpu     = esl_opt_GetInteger(go, "-t");
-  ESL_DSQDATA       *dd       = NULL;
-  ESL_DSQDATA_CHUNK *chu      = NULL;
+  ESL_GETOPTS       *go         = esl_getopts_CreateDefaultApp(options, 1, argc, argv, banner, usage);
+  ESL_ALPHABET      *abc        = NULL;
+  char              *basename   = esl_opt_GetArg(go, 1);
+  int                do_summary = esl_opt_GetBoolean(go, "-c");
+  int                do_resct   = esl_opt_GetBoolean(go, "-r");
+  int                ncpu       = 1;
+  ESL_DSQDATA       *dd         = NULL;
+  ESL_DSQDATA_CHUNK *chu        = NULL;
+  int                nchunk     = 0;
   int                i;
   int64_t            pos;
   int64_t            ct[128], total;
@@ -1760,18 +1780,25 @@ main(int argc, char **argv)
 
   for (x = 0; x < 127; x++) ct[x] = 0;
 
+  if (do_summary) esl_dataheader(stdout, 8, "idx", 4, "nseq", 8, "nres", 7, "npacket", 0);
+
   while ((status = esl_dsqdata_Read(dd, &chu)) == eslOK)
     {
-      if (! no_count)
+      nchunk++;
+      
+      if (do_resct)
 	for (i = 0; i < chu->N; i++)
 	  for (pos = 1; pos <= chu->L[i]; pos++)
 	    ct[ chu->dsq[i][pos] ]++;
 
+      if (do_summary)
+	printf("%-8d %4d %8" PRId64 " %7d\n", nchunk, chu->N, esl_vec_LSum(chu->L, chu->N), chu->pn);
+
       esl_dsqdata_Recycle(dd, chu);
     }
   if (status != eslEOF) esl_fatal("unexpected error %d in reading dsqdata", status);
 
-  if (! no_count)
+  if (do_resct)
     {
       total = 0;
       for (x = 0; x < abc->Kp; x++) 
