@@ -1,6 +1,6 @@
 /* Sequence and profile caches, used by the hmmpgmd daemon.
  */
-#include "p7_config.h"
+#include <p7_config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -39,7 +39,7 @@ p7_seqcache_Open(char *seqfile, P7_SEQCACHE **ret_cache, char *errbuf)
   int                val;
   int                status;
 
-  int32_t            seq_cnt;
+  uint64_t            seq_cnt;
   int32_t            db_cnt;
   int32_t            db_inx[32];
   uint32_t           db_key;
@@ -49,7 +49,7 @@ p7_seqcache_Open(char *seqfile, P7_SEQCACHE **ret_cache, char *errbuf)
   uint64_t           hdr_size;
 
   char              *hdr_ptr;
-  char              *res_ptr;
+  ESL_DSQ           *res_ptr;
   char              *desc_ptr;
   char              *ptr;
   char               buffer[512];
@@ -209,7 +209,7 @@ p7_seqcache_Open(char *seqfile, P7_SEQCACHE **ret_cache, char *errbuf)
   }
   if (status != eslEOF) { printf("Unexpected error %d at %d\n", status, inx); return status; }
 
-  if (inx != seq_cnt) { printf("inx:: %d %d\n", inx, seq_cnt); return eslEFORMAT; }
+  if (inx != seq_cnt) { printf("inx:: %d %" PRIu64 "\n", inx, seq_cnt);  return eslEFORMAT; }
   if (hdr_size != 0)  { printf("inx:: %d hdr %d\n", inx, (int)hdr_size); return eslEFORMAT; }
   if (res_size != 1)  { printf("inx:: %d size %d %d\n", inx, (int)sq->n + 1, (int)res_size); return eslEFORMAT; }
 
@@ -322,6 +322,7 @@ p7_seqcache_Close(P7_SEQCACHE *cache)
 
 #include "easel.h"
 #include "esl_alphabet.h"
+#include "esl_dsq.h"
 #include "esl_getopts.h"
 #include "esl_sq.h"
 #include "esl_sqio.h"
@@ -509,7 +510,7 @@ main(int argc, char **argv)
       fprintf(ofp, "\n");
 
       for (pos = 0; pos < info->n; pos += 60) {
-        esl_abc_TextizeN(abc, info->dsq+pos+1, 60, buf);
+        esl_dsq_TextizeN(abc, info->dsq+pos+1, 60, buf);
         buf[60] = '\0';
         fprintf(ofp, "%s\n", buf);
       }
