@@ -76,6 +76,7 @@ static ESL_OPTIONS options[] = {
   /* control of output(s) */
   { "-o",           eslARG_OUTFILE,      NULL, NULL, NULL,    NULL,  NULL,  NULL,          "direct output to file <f>, not stdout",                        4 },
   { "-A",           eslARG_OUTFILE,      NULL, NULL, NULL,    NULL,  NULL,  NULL,          "save multiple alignment of all hits to file <f>",              4 },
+  { "--Aformat",    eslARG_STRING,"Stockholm", NULL, NULL,    NULL,  "-A",  NULL,          "specify MSA output format <s> for -A",                         4 },
   { "--tblout",     eslARG_OUTFILE,      NULL, NULL, NULL,    NULL,  NULL,  NULL,          "save parseable table of hits to file <f>",                     4 },
   { "--dfamtblout", eslARG_OUTFILE,      NULL, NULL, NULL,    NULL,  NULL,  NULL,          "save table of hits to file, in Dfam format <f>",               4 },
   { "--aliscoresout", eslARG_OUTFILE,    NULL, NULL, NULL,    NULL,  NULL,  NULL,          "save scores for each position in each alignment to <f>",       4 },
@@ -182,6 +183,7 @@ struct cfg_s {
   FILE            *ofp;               // "human-readable" main results (stdout or -o)
   FILE            *tblfp;             // tabular results (optional; NULL or --tblout)
   FILE            *afp;               // multiple sequence alignment output (optional; NULL or -A)
+  int              Aformat;           // format of optional <afp> (default Stockholm)
   FILE            *dfamtblfp;         // tabular Dfam results table (optional; NULL or --dfamtblout)
   FILE            *aliscoresfp;       // alignment scores output (optional; NULL or --aliscoresout)
   FILE            *hmmoutfp;          // constructed profile HMMs, with --qseq|--qmsa (optional; NULL or --hmmout)
@@ -252,7 +254,7 @@ static void          assign_msa_name(struct cfg_s *cfg, ESL_MSA *msa);
 #ifdef p7ENABLE_FMINDEX
 static P7_SCOREDATA *create_fm_scoredata(struct cfg_s *cfg, P7_PROFILE *gm, P7_OPROFILE *om);
 #endif
-static void          output_optional_msa(FILE *afp, P7_HMM *hmm, P7_TOPHITS *th);
+static void          output_optional_msa(FILE *afp, const P7_HMM *hmm, const P7_TOPHITS *th, int outfmt);
 
 
 /*****************************************************************
@@ -536,7 +538,7 @@ main(int argc, char **argv)
 
       if (cfg.tblfp)       p7_tophits_TabularTargets(cfg.tblfp,       hmm->name, hmm->acc, info->th, info->pli, (nquery == 1));
       if (cfg.dfamtblfp)   p7_tophits_TabularXfam   (cfg.dfamtblfp,   hmm->name, hmm->acc, info->th, info->pli);
-      if (cfg.afp)         output_optional_msa      (cfg.afp,         hmm,       info->th); 
+      if (cfg.afp)         output_optional_msa      (cfg.afp,         hmm,                 info->th, cfg.Aformat); 
       if (cfg.aliscoresfp) p7_tophits_AliScores     (cfg.aliscoresfp, hmm->name, info->th);
       if (cfg.hmmoutfp)    p7_hmmfile_WriteASCII    (cfg.hmmoutfp, /*fmt=default*/-1, hmm);
 
@@ -912,6 +914,11 @@ initialize_cfg(ESL_GETOPTS *go, struct cfg_s *cfg)
   if (esl_opt_IsOn(go, "--aliscoresout"))  { if ((cfg->aliscoresfp  = fopen(esl_opt_GetString(go, "--aliscoresout"),"w")) == NULL) p7_Fail("Failed to open alignment scores output file %s for writing\n", esl_opt_GetString(go, "--aliscoresout")); }
   if (esl_opt_IsOn(go, "--hmmout"))        { if ((cfg->hmmoutfp     = fopen(esl_opt_GetString(go, "--hmmout"),      "w")) == NULL) p7_Fail("Failed to open query HMM output file %s for writing\n",        esl_opt_GetString(go, "--hmmout")); }
   
+  if (esl_opt_IsOn(go, "-A")) {
+    if ((cfg->Aformat  = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--Aformat"))) == eslMSAFILE_UNKNOWN)  // default "Stockholm" is set in OPTIONS
+      p7_Fail("%s is not a recognized MSA file format\n", esl_opt_GetString(go, "--Aformat"));
+  }
+
 #ifdef HMMER_THREADS
   cfg->ncpus = ESL_MIN(esl_opt_GetInteger(go, "--cpu"), esl_threads_GetCPUCount());
 #else
@@ -1472,19 +1479,19 @@ create_fm_scoredata(struct cfg_s *cfg, P7_PROFILE *gm, P7_OPROFILE *om)
 #endif
 
 static void
-output_optional_msa(FILE *afp, P7_HMM *hmm, P7_TOPHITS *th)
- {
-   ESL_MSA *msa;
+output_optional_msa(FILE *afp, const P7_HMM *hmm, const P7_TOPHITS *th, int outfmt)
+{
+  ESL_MSA *msa;
 
-   if ( p7_tophits_Alignment(th, hmm->abc, NULL, NULL, 0, p7_DEFAULT, &msa) == eslOK) 
-     {
-       esl_msa_SetName     (msa, hmm->name, -1);
-       esl_msa_SetAccession(msa, hmm->acc,  -1);
-       esl_msa_SetDesc     (msa, hmm->desc, -1);
-       esl_msa_FormatAuthor(msa, "nhmmer (HMMER %s)", HMMER_VERSION);
-
-       esl_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
-       esl_msa_Destroy(msa);
-     }
- }
+  if (p7_tophits_Alignment(th, hmm->abc, NULL, NULL, 0, p7_DEFAULT, &msa) == eslOK) // returns eslFAIL if there are no hits
+    {
+      esl_msa_SetName     (msa, hmm->name, -1);
+      esl_msa_SetAccession(msa, hmm->acc,  -1);
+      esl_msa_SetDesc     (msa, hmm->desc, -1);
+      esl_msa_FormatAuthor(msa, "nhmmer (HMMER %s)", HMMER_VERSION);
+  
+      esl_msafile_Write(afp, msa, outfmt);
+      esl_msa_Destroy(msa);
+    }
+}
 
