@@ -6,7 +6,7 @@
  *   3. Example 1: search mode (in a sequence db)
  *   4. Example 2: scan mode (in an HMM db)
  */
-#include "p7_config.h"
+#include <p7_config.h>
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -96,7 +96,7 @@ typedef struct {
  * Throws:    <NULL> on allocation failure.
  */
 P7_PIPELINE *
-p7_pipeline_Create(ESL_GETOPTS *go, int M_hint, int L_hint, int long_targets, enum p7_pipemodes_e mode)
+p7_pipeline_Create(const ESL_GETOPTS *go, int M_hint, int L_hint, int long_targets, enum p7_pipemodes_e mode)
 {
   P7_PIPELINE *pli  = NULL;
   int          seed = (go ? esl_opt_GetInteger(go, "--seed") : 42);
@@ -575,10 +575,27 @@ p7_pli_NewModelThresholds(P7_PIPELINE *pli, const P7_OPROFILE *om)
 int
 p7_pli_NewSeq(P7_PIPELINE *pli, const ESL_SQ *sq)
 {
-  if (!pli->long_targets) pli->nseqs++; // if long_targets, sequence counting happens in the serial loop, which can track multiple windows for a single long sequence
-  pli->nres += sq->n;
-  if (pli->Z_setby == p7_ZSETBY_NTARGETS && pli->mode == p7_SEARCH_SEQS) pli->Z = pli->nseqs;
+  if (! pli->long_targets) {  // if long_targets, sequence counting happens in main loop of the master thread, which can track multiple windows for a single long sequence
+    pli->nseqs++;             
+    if (pli->Z_setby == p7_ZSETBY_NTARGETS && pli->mode == p7_SEARCH_SEQS) pli->Z = pli->nseqs; //  ... whereas worker threads call p7_pli_NewSeq(), so pli->nseqs can't even be read, when in longtargets mode
+  }
+  pli->nres += sq->n;  
   return eslOK;
+
+  // Note on what nhmmer (long_targets mode) is doing w.r.t. threads and nseqs/nres.
+  //
+  // nhmmer counts pli->nseqs in the master thread (albeit with a possible
+  // adjustment at the end in FM-indexing mode). nres, though, is
+  // counted by *worker* threads, by calling p7_pli_NewSeq(), followed
+  // by adjustments in pipeline_thread() for window overlap and the
+  // opposite strand.  
+  //
+  // To avoid thread races, in longtarget mode you must make sure
+  // you only touch nres here, not nseqs. Since nhmmer does not use
+  // nseqs for E-value calculations (it uses nres/W) and furthermore
+  // it does not use lower bound E-values during a search the way
+  // hmmsearch does, we can just have p7_pli_NewSeq() update nres
+  // and ignore anything having to do with nseqs.
 }
 
 /* Function:  p7_pipeline_Merge()
@@ -638,18 +655,18 @@ p7_pipeline_Merge(P7_PIPELINE *p1, P7_PIPELINE *p2)
   return eslOK;
 }
 
-/* Function:  p7_Pipeline()
- * Synopsis:  HMMER3's accelerated seq/profile comparison pipeline.
+
+/* Function:  p7_Pipeline_Overthruster()
+ * Synopsis:  The filters of HMMER3's accelerated seq/profile comparison pipeline.
  *
- * Purpose:   Run H3's accelerated pipeline to compare profile <om>
- *            against sequence <sq>. If a significant hit is found,
- *            information about it is added to the <hitlist>. The pipeline 
- *            accumulates beancounting information about how many comparisons
- *            flow through the pipeline while it's active.
- *            
- * Returns:   <eslOK> on success. If a significant hit is obtained,
- *            its information is added to the growing <hitlist>. 
- *            
+ * Purpose:   Run H3's comparison filters to determine if a sequence is sufficiently
+ *            similar to an HMM to justify running the third stage.
+ * 
+ * Returns:   <eslOK> if the sequence and HMM are similar enough that the   
+ *            main stage should be run.
+ *            <eslFAIL> if the sequence and HMM are not similar enough that
+ *            the main stage should be run.
+ * 
  *            <eslEINVAL> if (in a scan pipeline) we're supposed to
  *            set GA/TC/NC bit score thresholds but the model doesn't
  *            have any.
@@ -663,81 +680,167 @@ p7_pipeline_Merge(P7_PIPELINE *p1, P7_PIPELINE *p2)
  *
  * Throws:    <eslEMEM> on allocation failure.
  *
+ *            <eslETYPE> if <sq> is more than 100K long, which can
+ *            happen when someone uses hmmsearch/hmmscan instead of
+ *            nhmmer/nhmmscan on a genome DNA seq db.
+ *
  * Xref:      J4/25.
+ *
+ * Note:      Error handling needs improvement. The <eslETYPE> exception
+ *            was added as a late bugfix. It really should be an <eslEINVAL>
+ *            normal error (because it's a user error). But then we need
+ *            all our p7_Pipeline() calls to check their return status
+ *            and handle normal errors appropriately, which we haven't 
+ *            been careful enough about. [SRE H9/4]
  */
-int
-p7_Pipeline(P7_PIPELINE *pli, P7_OPROFILE *om, P7_BG *bg, const ESL_SQ *sq, const ESL_SQ *ntsq, P7_TOPHITS *hitlist)
+extern int p7_Pipeline_Overthruster(P7_PIPELINE *pli, P7_OPROFILE *om, P7_BG *bg, const ESL_SQ *sq, float *ret_fwdsc, float *ret_nullsc)
 {
-  P7_HIT          *hit     = NULL;     /* ptr to the current hit output data      */
-  float            usc, vfsc, fwdsc;   /* filter scores                           */
-  float            filtersc;           /* HMM null filter score                   */
-  float            nullsc;             /* null model score                        */
-  float            seqbias;  
-  float            seq_score;          /* the corrected per-seq bit score */
-  float            sum_score;           /* the corrected reconstruction score for the seq */
-  float            pre_score, pre2_score; /* uncorrected bit scores for seq */
-  double           P;                /* P-value of a hit */
-  double           lnP;              /* log P-value of a hit */
-  int              Ld;               /* # of residues in envelopes */
-  int              d;
-  int              status;
-  
-  if (sq->n == 0) return eslOK;    /* silently skip length 0 seqs; they'd cause us all sorts of weird problems */
+  //P7_HIT *hit = NULL;     /* ptr to the current hit output data      */
+  float usc, vfsc, fwdsc; /* filter scores                           */
+  float filtersc;         /* HMM null filter score                   */
+  float nullsc;           /* null model score                        */
+  //float seqbias;
+  float seq_score;             /* the corrected per-seq bit score */
+  //float sum_score;             /* the corrected reconstruction score for the seq */
+  //float pre_score, pre2_score; /* uncorrected bit scores for seq */
+  double P;                    /* P-value of a hit */
+  //double lnP;                  /* log P-value of a hit */
+  //int Ld;                      /* # of residues in envelopes */
+  //int d;
+  int status;
 
-  p7_omx_GrowTo(pli->oxf, om->M, 0, sq->n);    /* expand the one-row omx if needed */
+  if (sq->n == 0)
+    return eslFAIL; /* silently skip length 0 seqs; they'd cause us all sorts of weird problems */
+  if (sq->n > 100000)
+    ESL_EXCEPTION(eslETYPE, "Target sequence length > 100K, over comparison pipeline limit.\n(Did you mean to use nhmmer/nhmmscan?)");
+
+  p7_omx_GrowTo(pli->oxf, om->M, 0, sq->n); /* expand the one-row omx if needed */
 
   /* Base null model score (we could calculate this in NewSeq(), for a scan pipeline) */
-  p7_bg_NullOne  (bg, sq->dsq, sq->n, &nullsc);
+  p7_bg_NullOne(bg, sq->dsq, sq->n, &nullsc);
 
   /* First level filter: the MSV filter, multihit with <om> */
   p7_MSVFilter(sq->dsq, sq->n, om, pli->oxf, &usc);
   seq_score = (usc - nullsc) / eslCONST_LOG2;
-  P = esl_gumbel_surv(seq_score,  om->evparam[p7_MMU],  om->evparam[p7_MLAMBDA]);
-  if (P > pli->F1) return eslOK;
+  P = esl_gumbel_surv(seq_score, om->evparam[p7_MMU], om->evparam[p7_MLAMBDA]);
+  if (P > pli->F1)
+    return eslFAIL;
   pli->n_past_msv++;
 
   /* biased composition HMM filtering */
   if (pli->do_biasfilter)
-    {
-      p7_bg_FilterScore(bg, sq->dsq, sq->n, &filtersc);
-      seq_score = (usc - filtersc) / eslCONST_LOG2;
-      P = esl_gumbel_surv(seq_score,  om->evparam[p7_MMU],  om->evparam[p7_MLAMBDA]);
-      if (P > pli->F1) return eslOK;
-    }
-  else filtersc = nullsc;
+  {
+    p7_bg_FilterScore(bg, sq->dsq, sq->n, &filtersc);
+    seq_score = (usc - filtersc) / eslCONST_LOG2;
+    P = esl_gumbel_surv(seq_score, om->evparam[p7_MMU], om->evparam[p7_MLAMBDA]);
+    if (P > pli->F1)
+      return eslFAIL;
+  }
+  else
+    filtersc = nullsc;
   pli->n_past_bias++;
 
   /* In scan mode, if it passes the MSV filter, read the rest of the profile */
   if (pli->mode == p7_SCAN_MODELS)
-    {
-      if (pli->hfp) p7_oprofile_ReadRest(pli->hfp, om);
-      p7_oprofile_ReconfigRestLength(om, sq->n);
-      if ((status = p7_pli_NewModelThresholds(pli, om)) != eslOK) return status; /* pli->errbuf has err msg set */
-    }
+  {
+    if (pli->hfp)
+      p7_oprofile_ReadRest(pli->hfp, om);
+    p7_oprofile_ReconfigRestLength(om, sq->n);
+    if ((status = p7_pli_NewModelThresholds(pli, om)) != eslOK)
+      return status; /* pli->errbuf has err msg set */
+  }
 
   /* Second level filter: ViterbiFilter(), multihit with <om> */
   if (P > pli->F2)
-    {
-      p7_ViterbiFilter(sq->dsq, sq->n, om, pli->oxf, &vfsc);  
-      seq_score = (vfsc-filtersc) / eslCONST_LOG2;
-      P  = esl_gumbel_surv(seq_score,  om->evparam[p7_VMU],  om->evparam[p7_VLAMBDA]);
-      if (P > pli->F2) return eslOK;
-    }
+  {
+    p7_ViterbiFilter(sq->dsq, sq->n, om, pli->oxf, &vfsc);
+    seq_score = (vfsc - filtersc) / eslCONST_LOG2;
+    P = esl_gumbel_surv(seq_score, om->evparam[p7_VMU], om->evparam[p7_VLAMBDA]);
+    if (P > pli->F2)
+      return eslFAIL;
+  }
   pli->n_past_vit++;
-
-
+/*  for(int npc=0; npc < bg->fhmm->M; npc++){
+    fprintf(stderr, "%f, ", bg->fhmm->pi[npc]);
+  }
+  for(int npc0 = 0; npc0< (bg->fhmm->M * (bg->fhmm->M +1)); npc0++){
+    fprintf(stderr, "%f, ",bg->fhmm->t[npc0]);
+  } 
+  for(int npc1 = 0; npc1 < (bg->fhmm->M * bg->fhmm->abc->K); npc1++){
+    fprintf(stderr, "%f, ", bg->fhmm->e[npc1]);
+  }
+  for(int npc2 = 0; npc2 < (bg->fhmm->M * bg->fhmm->abc->Kp); npc2++){
+    fprintf(stderr, "%f, ", bg->fhmm->eo[npc2]);
+  }
+  fprintf(stderr, "%f\n", bg->p1);*/
+  //fprintf(stderr, "%d \n", sq->n);
   /* Parse it with Forward and obtain its real Forward score. */
   p7_ForwardParser(sq->dsq, sq->n, om, pli->oxf, &fwdsc);
-  seq_score = (fwdsc-filtersc) / eslCONST_LOG2;
-  P = esl_exp_surv(seq_score,  om->evparam[p7_FTAU],  om->evparam[p7_FLAMBDA]);
-  if (P > pli->F3) return eslOK;
+  seq_score = (fwdsc - filtersc) / eslCONST_LOG2;
+ //fprintf(stderr, "%s, %f, %f, %f\n", sq->name, fwdsc, filtersc, nullsc);
+ //fprintf(stderr, "%f, %f\n", bg->p1, bg->omega);
+  P = esl_exp_surv(seq_score, om->evparam[p7_FTAU], om->evparam[p7_FLAMBDA]);
+  if (P > pli->F3)
+    return eslFAIL;
   pli->n_past_fwd++;
+  *ret_fwdsc = fwdsc;
+  *ret_nullsc = nullsc;
+  return eslOK; // if we get this far, we passed all the filters and should proceed to the main stage
+}
 
-  /* ok, it's for real. Now a Backwards parser pass, and hand it to domain definition workflow */
+  /* Function:  p7_Pipeline_Mainstage()
+   * Synopsis:  HMMER3's accelerated seq/profile comparison pipeline.
+   *
+   * Purpose:   Run the main stage of HMMER's accelerated comparison
+   *            pipeline to determine if a hit really has occurred  
+   *            and update the hitlist accordingly.
+   *
+   * Returns:   <eslOK> on success. If a significant hit is obtained,
+   *            its information is added to the growing <hitlist>.
+   *
+   *            <eslEINVAL> if (in a scan pipeline) we're supposed to
+   *            set GA/TC/NC bit score thresholds but the model doesn't
+   *            have any.
+   *
+   *            <eslERANGE> on numerical overflow errors in the
+   *            optimized vector implementations; particularly in
+   *            posterior decoding. I don't believe this is possible for
+   *            multihit local models, but I'm set up to catch it
+   *            anyway. We may emit a warning to the user, but cleanly
+   *            skip the problematic sequence and continue.
+   *
+   * Throws:    <eslEMEM> on allocation failure.
+   *
+   *            <eslETYPE> if <sq> is more than 100K long, which can
+   *            happen when someone uses hmmsearch/hmmscan instead of
+   *            nhmmer/nhmmscan on a genome DNA seq db.
+   *
+   * Xref:      J4/25.
+   *
+   * Note:      Error handling needs improvement. The <eslETYPE> exception
+   *            was added as a late bugfix. It really should be an <eslEINVAL>
+   *            normal error (because it's a user error). But then we need
+   *            all our p7_Pipeline() calls to check their return status
+   *            and handle normal errors appropriately, which we haven't
+   *            been careful enough about. [SRE H9/4]
+   */
+  extern int p7_Pipeline_Mainstage(P7_PIPELINE * pli, P7_OPROFILE * om, P7_BG * bg, const ESL_SQ *sq, const ESL_SQ *ntsq, P7_TOPHITS *hitlist, float fwdsc, float nullsc)
+  {
+    P7_HIT *hit = NULL;     /* ptr to the current hit output data      */                    
+    float seqbias;
+    float seq_score;             /* the corrected per-seq bit score */
+    float sum_score;             /* the corrected reconstruction score for the seq */
+    float pre_score, pre2_score; /* uncorrected bit scores for seq */
+    double lnP;                  /* log P-value of a hit */
+    int Ld;                      /* # of residues in envelopes */
+    int d;
+    int status;
+
+  /* Run a Backwards parser pass, and hand it to domain definition workflow */
   p7_omx_GrowTo(pli->oxb, om->M, 0, sq->n);
   p7_BackwardParser(sq->dsq, sq->n, om, pli->oxf, pli->oxb, NULL);
-
-  status = p7_domaindef_ByPosteriorHeuristics(sq, ntsq, om, pli->oxf, pli->oxb, pli->fwd, pli->bck, pli->ddef, bg, FALSE, NULL, NULL, NULL);
+ 
+    status = p7_domaindef_ByPosteriorHeuristics(sq, ntsq, om, pli->oxf, pli->oxb, pli->fwd, pli->bck, pli->ddef, bg, FALSE, NULL, NULL, NULL);
   if (status != eslOK) ESL_FAIL(status, pli->errbuf, "domain definition workflow failure"); /* eslERANGE can happen  */
   if (pli->ddef->nregions   == 0) return eslOK; /* score passed threshold but there's no discrete domains here       */
   if (pli->ddef->nenvelopes == 0) return eslOK; /* rarer: region was found, stochastic clustered, no envelopes found */
@@ -906,7 +1009,36 @@ p7_Pipeline(P7_PIPELINE *pli, P7_OPROFILE *om, P7_BG *bg, const ESL_SQ *sq, cons
   return eslOK;
 }
 
+/* Function:  p7_Pipeline()
+ * Synopsis:  HMMER3's accelerated seq/profile comparison pipeline.
+ *
+ * Purpose:   This function is now just a wrapper around the 
+ *            p7_pipeline_Overthruster() and 
+ *            p7_pipeline_Mainstage() calls: it first calls p7_pipeline_Overthruster(),
+ *            to run HMMER's filter stages on the sequence/HMM comparison. If the 
+ *            filters score highly enough that the main stage should be run, it
+ *            calls p7_pipeline_Mainstage for final hit/miss determination and hitlist
+ *            insertion.
+ */
+extern int p7_Pipeline(P7_PIPELINE *pli, P7_OPROFILE *om, P7_BG *bg, const ESL_SQ *sq, const ESL_SQ *ntsq, P7_TOPHITS *hitlist)
+{
+  int status;
+  float fwdsc;
+  float nullsc;
 
+  status = p7_Pipeline_Overthruster(pli, om, bg, sq, &fwdsc, &nullsc);
+  if (status == eslOK){ //run the main stage
+    return (p7_Pipeline_Mainstage(pli, om, bg, sq, ntsq, hitlist, fwdsc, nullsc));
+  }
+  if (status == eslFAIL){  /* overthruster status of eslFAIL indicates that the thruster completed 
+                              correctly, but the comparison did not score highly enough to proceed,
+                              so return a correct completion of the pipeline */
+    return eslOK;
+  }
+  else{
+    return status;
+  }
+}
 
 /* Function:  p7_pli_computeAliScores()
  * Synopsis:  Compute per-position scores for the alignment for a domain
@@ -1036,8 +1168,7 @@ static int
 p7_pli_postViterbi_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_BG *bg, P7_TOPHITS *hitlist, const P7_SCOREDATA *data,
     int64_t seqidx, int window_start, int window_len, ESL_DSQ *subseq,
     int64_t seq_start, char *seq_name, char *seq_source, char* seq_acc, char* seq_desc, int seq_len,
-    int complementarity, int *overlap, P7_PIPELINE_LONGTARGET_OBJS *pli_tmp
-)
+    int complementarity, int *overlap, P7_PIPELINE_LONGTARGET_OBJS *pli_tmp)
 {
   P7_DOMAIN        *dom     = NULL;     /* convenience variable, ptr to current domain */
   P7_HIT           *hit     = NULL;     /* ptr to the current hit output data      */
@@ -1137,7 +1268,7 @@ p7_pli_postViterbi_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_BG *bg, P7_T
         continue; // anything less than this is a funny byproduct of the Forward score passing a very low threshold, but no reliable alignment existing that supports it
       }
 
-     /* note: this bitscore was computed under a model with length of
+     /* Note: this bitscore was computed under a model with length of
       * env_len (jenv-ienv+1). Here, the score is modified (reduced) by
       * treating the hit as though it came from a window of length
       * om->max_length. To do this:
@@ -1471,16 +1602,14 @@ p7_Pipeline_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_SCOREDATA *data,
                         const FM_DATA *fmf, const FM_DATA *fmb, FM_CFG *fm_cfg
                         )
 {
-  int              i;
-  int              status;
-  float            nullsc;   /* null model score                        */
-  float            usc;      /* msv score  */
+  float            nullsc;   // null model score
+  float            usc;      // msv score
   float            P;
-  float            bias_filtersc;
-
+  //  float            bias_filtersc;  // SRE: see comment below on TJW's non-use of this
   ESL_DSQ          *subseq;
   uint64_t         seq_start;
-
+  int              i;
+  int              status;
 
   P7_HMM_WINDOWLIST msv_windowlist;
   P7_HMM_WINDOWLIST vit_windowlist;
@@ -1499,6 +1628,13 @@ p7_Pipeline_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_SCOREDATA *data,
   ESL_ALLOC(pli_tmp->scores, sizeof(float) * om->abc->Kp * 4); //allocation of space to store scores that will be used in p7_oprofile_Update(Fwd|Vit|MSV)EmissionScores
   ESL_ALLOC(pli_tmp->fwd_emissions_arr, sizeof(float) *  om->abc->Kp * (om->M+1));
 
+  memset(pli_tmp->fwd_emissions_arr, 0, sizeof(float) *  om->abc->Kp * (om->M+1));
+  // ^^ note on why the memset() is there:
+  // p7_oprofile_GetFwdEmissionsScoreArray() appears problematic.
+  // iss #320 detected use of uninitialized memory and I'm not surprised; probably not the only thing wrong.
+  // the memset() above is added to patch iss #320, but I expect other more subtle problems. See note on the function.
+  // [SRE 2024/0107-h3-iss320]
+
   msv_windowlist.windows = NULL;
   vit_windowlist.windows = NULL;
   p7_hmmwindow_init(&msv_windowlist);
@@ -1516,7 +1652,7 @@ p7_Pipeline_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_SCOREDATA *data,
    * short high-scoring regions.
    */
   if (fmf) // using an FM-index
-    p7_SSVFM_longlarget(om, 2.0, bg, pli->F1, fmf, fmb, fm_cfg, data, pli->strands, &msv_windowlist );
+    p7_SSVFM_longlarget(om, 2.0, bg, pli->F1, fmf, fmb, fm_cfg, data, pli->strands, pli->r, &msv_windowlist );   // SRE: ** _longlarget ** ????
   else // compare directly to sequence
     p7_SSVFilter_longtarget(sq->dsq, sq->n, om, pli->oxf, data, bg, pli->F1, &msv_windowlist);
 
@@ -1525,21 +1661,18 @@ p7_Pipeline_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_SCOREDATA *data,
    */
   if ( msv_windowlist.count > 0 ) {
 
-    /* In scan mode, if it passes the MSV filter, read the rest of the profile
-     * Not necessary for dummy mode, where the ->base_w variable checks cause compilation failure*/
-#ifndef P7_IMPL_DUMMY_INCLUDED
+    /* In scan mode, if it passes the MSV filter, read the rest of the profile */
     if (!fmf && pli->hfp)
-    {
-      if (om->base_w == 0 &&  om->scale_w == 0) { // we haven't already read this hmm (if we're on the second strand, we would've)
-        p7_oprofile_ReadRest(pli->hfp, om);
-        if ((status = p7_pli_NewModelThresholds(pli, om)) != eslOK) goto ERROR;
+      {
+	if (om->base_w == 0 &&  om->scale_w == 0) { // we haven't already read this hmm (if we're on the second strand, we would've)
+	  p7_oprofile_ReadRest(pli->hfp, om);
+	  if ((status = p7_pli_NewModelThresholds(pli, om)) != eslOK) goto ERROR;
+	}
       }
-    }
-#endif
 
     p7_oprofile_GetFwdEmissionArray(om, bg, pli_tmp->fwd_emissions_arr);
 
-    if (data->prefix_lengths == NULL)  //otherwise, already filled in
+    if (data->prefix_lengths == NULL)  // otherwise, already filled in
       p7_hmm_ScoreDataComputeRest(om, data);
 
     p7_pli_ExtendAndMergeWindows (om, data, &msv_windowlist, 0);
@@ -1607,7 +1740,18 @@ p7_Pipeline_LongTarget(P7_PIPELINE *pli, P7_OPROFILE *om, P7_SCOREDATA *data,
       p7_bg_SetLength(bg, window->length);
       p7_bg_NullOne  (bg, subseq, window->length, &nullsc);
 
-      p7_bg_FilterScore(bg, subseq, window->length, &bias_filtersc);
+      /* SRE: Unclear what TJW's intent here is.
+       * He was calling p7_bg_FilterScore() without checking pli->do_biasfilter,
+       * which is a bug: p7_pli_NewModel only initializes the filter HMM when
+       * pli->do_biasfilter is TRUE, so memory sanitizers will see uninitialized
+       * memory being used. But he doesn't use bias_filtersc for anything,
+       * so it's a no-op anyway. I'm going to:
+       *    1. fix the code by checking for pli->do_biasfilter
+       *    2. then comment it out because he isn't using the result anyway.
+       */
+      // if (pli->do_biasfilter)
+      //   p7_bg_FilterScore(bg, subseq, window->length, &bias_filtersc);
+
       // Compute standard MSV to ensure that bias doesn't overcome SSV score when MSV
       // would have survived it
       p7_oprofile_ReconfigMSVLength(om, window->length);
@@ -1701,45 +1845,37 @@ p7_pli_Statistics(FILE *ofp, P7_PIPELINE *pli, ESL_STOPWATCH *w)
   fprintf(ofp, "Internal pipeline statistics summary:\n");
   fprintf(ofp, "-------------------------------------\n");
   if (pli->mode == p7_SEARCH_SEQS) {
-    fprintf(ofp, "Query model(s):                %15" PRId64 "  (%" PRId64 " nodes)\n",     pli->nmodels, pli->nnodes);
-    fprintf(ofp, "Target sequences:              %15" PRId64 "  (%" PRId64 " residues searched)\n",  pli->nseqs,   pli->nres);
+    fprintf(ofp, "Query model(s):              %15" PRId64 "  (%" PRId64 " nodes)\n",     pli->nmodels, pli->nnodes);
+    fprintf(ofp, "Target sequences:            %15" PRId64 "  (%" PRId64 " residues searched)\n",  pli->nseqs,   pli->nres);
     ntargets = pli->nseqs;
   } else {
-    fprintf(ofp,   "Query sequence(s):           %15" PRId64 "  (%" PRId64 " residues searched)\n",  pli->nseqs,   pli->nres);
-    fprintf(ofp,   "Target model(s):             %15" PRId64 "  (%" PRId64 " nodes)\n",     pli->nmodels, pli->nnodes);
+    fprintf(ofp, "Query sequence(s):           %15" PRId64 "  (%" PRId64 " residues searched)\n",  pli->nseqs,   pli->nres);
+    fprintf(ofp, "Target model(s):             %15" PRId64 "  (%" PRId64 " nodes)\n",     pli->nmodels, pli->nnodes);
     ntargets = pli->nmodels;
   }
 
-  if (pli->long_targets) {
-      fprintf(ofp, "Residues passing SSV filter:   %15" PRId64 "  (%.3g); expected (%.3g)\n",
-      //fprintf(ofp, "Windows passing MSV filter:   %15" PRId64 "  (%.4g); expected (%.4g)\n",
-          //pli->n_past_msv,
+  if (pli->long_targets) { // nhmmer style
+      fprintf(ofp, "Residues passing SSV filter: %15" PRId64 "  (%.3g); expected (%.3g)\n",
           pli->pos_past_msv,
           (double)pli->pos_past_msv / (pli->nres*pli->nmodels) ,
           pli->F1);
 
-      fprintf(ofp, "Residues passing bias filter:  %15" PRId64 "  (%.3g); expected (%.3g)\n",
-      //fprintf(ofp, "Windows passing bias filter:  %15" PRId64 "  (%.4g); expected (%.4g)\n",
-          //pli->n_past_bias,
+      fprintf(ofp, "Residues passing bias filter:%15" PRId64 "  (%.3g); expected (%.3g)\n",
           pli->pos_past_bias,
           (double)pli->pos_past_bias / (pli->nres*pli->nmodels) ,
           pli->F1);
 
-      fprintf(ofp, "Residues passing Vit filter:   %15" PRId64 "  (%.3g); expected (%.3g)\n",
-      //fprintf(ofp, "Windows passing Vit filter:   %15" PRId64 "  (%.4g); expected (%.4g)\n",
-          //pli->n_past_vit,
+      fprintf(ofp, "Residues passing Vit filter: %15" PRId64 "  (%.3g); expected (%.3g)\n",
           pli->pos_past_vit,
           (double)pli->pos_past_vit / (pli->nres*pli->nmodels) ,
           pli->F2);
 
-      fprintf(ofp, "Residues passing Fwd filter:   %15" PRId64 "  (%.3g); expected (%.3g)\n",
-      //fprintf(ofp, "Windows passing Fwd filter:   %15" PRId64 "  (%.4g); expected (%.4g)\n",
-          //pli->n_past_fwd,
+      fprintf(ofp, "Residues passing Fwd filter: %15" PRId64 "  (%.3g); expected (%.3g)\n",
           pli->pos_past_fwd,
           (double)pli->pos_past_fwd / (pli->nres*pli->nmodels) ,
           pli->F3);
 
-      fprintf(ofp, "Total number of hits:          %15d  (%.3g)\n",
+      fprintf(ofp, "Total number of hits:        %15d  (%.3g)\n",
           (int)pli->n_output,
           (double)pli->pos_output / (pli->nres*pli->nmodels) );
 
@@ -1793,7 +1929,7 @@ p7_pli_Statistics(FILE *ofp, P7_PIPELINE *pli, ESL_STOPWATCH *w)
  * ./pipeline_example <hmmfile> <sqfile>
  */
 
-#include "p7_config.h"
+#include <p7_config.h>
 
 #include "easel.h"
 #include "esl_getopts.h"
@@ -1849,8 +1985,8 @@ main(int argc, char **argv)
   p7_FLogsumInit();
 
   /* Read in one HMM */
-  if (p7_hmmfile_OpenE(hmmfile, NULL, &hfp, NULL) != eslOK) p7_Fail("Failed to open HMM file %s", hmmfile);
-  if (p7_hmmfile_Read(hfp, &abc, &hmm)            != eslOK) p7_Fail("Failed to read HMM");
+  if (p7_hmmfile_Open(hmmfile, NULL, &hfp, NULL) != eslOK) p7_Fail("Failed to open HMM file %s", hmmfile);
+  if (p7_hmmfile_Read(hfp, &abc, &hmm)           != eslOK) p7_Fail("Failed to read HMM");
   p7_hmmfile_Close(hfp);
 
   /* Open a sequence file */
@@ -1932,7 +2068,7 @@ main(int argc, char **argv)
  * ./pipeline_example2 <hmmdb> <sqfile>
  */
 
-#include "p7_config.h"
+#include <p7_config.h>
 
 #include "easel.h"
 #include "esl_getopts.h"
@@ -1994,7 +2130,7 @@ main(int argc, char **argv)
   esl_sqfile_Close(sqfp);
 
   /* Open the HMM db */
-  if (p7_hmmfile_OpenE(hmmfile, NULL, &hfp, NULL) != eslOK) p7_Fail("Failed to open HMM file %s", hmmfile);
+  if (p7_hmmfile_Open(hmmfile, NULL, &hfp, NULL) != eslOK) p7_Fail("Failed to open HMM file %s", hmmfile);
 
   /* Create a pipeline for the query sequence in scan mode */
   pli      = p7_pipeline_Create(go, 100, sq->n, FALSE, p7_SCAN_MODELS);
