@@ -1,6 +1,6 @@
 /* phmmer: search a protein sequence against a protein database
  */
-#include "p7_config.h"
+#include <p7_config.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,18 +54,13 @@ typedef struct {
 #define MPIOPTS     NULL
 #endif
 
-#ifdef HMMER_MPI
-#define DAEMONOPTS  "-o,-A,--tblout,--domtblout,--pfamtblout,--mpi,--stall"
-#else
-#define DAEMONOPTS  "-o,-A,--tblout,--domtblout,--pfamtblout"
-#endif
-
 static ESL_OPTIONS options[] = {
   /* name           type              default   env  range   toggles   reqs   incomp                             help                                       docgroup*/
   { "-h",           eslARG_NONE,        FALSE, NULL, NULL,      NULL,  NULL,  NULL,              "show brief help on version and usage",                         1 },
 /* Control of output */
   { "-o",           eslARG_OUTFILE,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "direct output to file <f>, not stdout",                        2 },
   { "-A",           eslARG_OUTFILE,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "save multiple alignment of hits to file <f>",                  2 },
+  { "--Aformat",    eslARG_STRING,"Stockholm", NULL, NULL,      NULL,  "-A",  NULL,              "specify MSA output format <s> for -A",                         2 },
   { "--tblout",     eslARG_OUTFILE,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "save parseable table of per-sequence hits to file <f>",        2 },
   { "--domtblout",  eslARG_OUTFILE,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "save parseable table of per-domain hits to file <f>",          2 },
   { "--pfamtblout", eslARG_OUTFILE,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "save table of hits and domains to file, in Pfam format <f>",   2 },
@@ -113,8 +108,6 @@ static ESL_OPTIONS options[] = {
   { "--seed",       eslARG_INT,         "42",  NULL, "n>=0",    NULL,  NULL,  NULL,              "set RNG seed to <n> (if 0: one-time arbitrary seed)",         12 },
   { "--qformat",    eslARG_STRING,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "assert query <seqfile> is in format <s>: no autodetection",   12 },
   { "--tformat",    eslARG_STRING,      NULL, NULL, NULL,      NULL,  NULL,  NULL,              "assert target <seqdb> is in format <s>>: no autodetection",   12 },
-  { "--daemon",     eslARG_NONE,        NULL, NULL, NULL,      NULL,  NULL,  DAEMONOPTS,        "run program as a daemon",                                     12 },
-
 #ifdef HMMER_THREADS
   { "--cpu",        eslARG_INT,  p7_NCPU,"HMMER_NCPU", "n>=0",NULL,  NULL,  CPUOPTS,            "number of parallel CPU workers to use for multithreads",      12 },
 #endif
@@ -295,7 +288,6 @@ output_header(FILE *ofp, ESL_GETOPTS *go, char *qfile, char *dbfile)
   }
   if (esl_opt_IsUsed(go, "--qformat")   && fprintf(ofp, "# query <seqfile> format asserted: %s\n",            esl_opt_GetString(go, "--qformat"))    < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
   if (esl_opt_IsUsed(go, "--tformat")   && fprintf(ofp, "# target <seqdb> format asserted:  %s\n",            esl_opt_GetString(go, "--tformat"))    < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-  if (esl_opt_IsUsed(go, "--daemon")    && fprintf(ofp, "run as a daemon process\n")                                                                 < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
 #ifdef HMMER_THREADS
   if (esl_opt_IsUsed(go, "--cpu")       && fprintf(ofp, "# number of worker threads:        %d\n",            esl_opt_GetInteger(go, "--cpu"))       < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");  
 #endif
@@ -405,6 +397,7 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   int              nquery   = 0;
   int              seed;
   int              textw;
+  int              Aformat  = eslMSAFILE_UNKNOWN;
   int              status   = eslOK;
   int              qstatus  = eslOK;
   int              sstatus  = eslOK;
@@ -424,7 +417,7 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   textw   = (esl_opt_GetBoolean(go, "--notextw") ? 0 : esl_opt_GetInteger(go, "--textw"));
   bg      = p7_bg_Create(abc);
 
-  /* If caller declared input formats, decode them */
+  /* If caller declared input/output formats, decode them */
   if (esl_opt_IsOn(go, "--qformat")) {
     qformat = esl_sqio_EncodeFormat(esl_opt_GetString(go, "--qformat"));
     if (qformat == eslSQFILE_UNKNOWN) p7_Fail("%s is not a recognized input sequence file format\n", esl_opt_GetString(go, "--qformat"));
@@ -433,17 +426,9 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
     dbformat = esl_sqio_EncodeFormat(esl_opt_GetString(go, "--tformat"));
     if (dbformat == eslSQFILE_UNKNOWN) p7_Fail("%s is not a recognized sequence database file format\n", esl_opt_GetString(go, "--tformat"));
   }
+  if ((Aformat  = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--Aformat"))) == eslMSAFILE_UNKNOWN)  // default "Stockholm" is set in OPTIONS
+    p7_Fail("%s is not a recognized MSA file format\n", esl_opt_GetString(go, "--Aformat"));
 
-  /* validate options if running as a daemon */
-  if (esl_opt_IsOn(go, "--daemon")) 
-    {
-      /* running as a daemon, the input format must be type daemon */
-      if (qformat != eslSQFILE_UNKNOWN && qformat != eslSQFILE_DAEMON) 
-	p7_Fail("Input format %s not supported.  Must be daemon\n", esl_opt_GetString(go, "--qformat"));
-      qformat = eslSQFILE_DAEMON;
-      
-      if (strcmp(cfg->qfile, "-") != 0) p7_Fail("Query sequence file must be '-', in daemon mode\n");
-    }
 
   /* Initialize a default builder configuration,
    * then set only the options we need for single sequence search
@@ -508,8 +493,8 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
     }
 #endif
 
-  infocnt = (ncpus == 0) ? 1 : ncpus;
-  ESL_ALLOC(info, sizeof(*info) * infocnt);
+  infocnt = (ncpus <= 0) ? 1 : ncpus;    
+  ESL_ALLOC(info, (ptrdiff_t) sizeof(*info) * infocnt); 
 
   /* Show header output */
   output_header(ofp, go, cfg->qfile, cfg->dbfile);
@@ -646,14 +631,13 @@ serial_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 	    if (qsq->acc[0]  != '\0') esl_msa_SetAccession(msa, qsq->acc,  -1);
 	    if (qsq->desc[0] != '\0') esl_msa_SetDesc     (msa, qsq->desc, -1);
 	    esl_msa_FormatAuthor(msa, "phmmer (HMMER %s)", HMMER_VERSION);
+            
+            esl_msafile_Write(afp, msa, Aformat);
 
-	    if (textw > 0) esl_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
-	    else           esl_msafile_Write(afp, msa, eslMSAFILE_PFAM);
-
-	    if (fprintf(ofp, "# Alignment of %d hits satisfying inclusion thresholds saved to: %s\n", msa->nseq, esl_opt_GetString(go, "-A")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-	  }
+            if (fprintf(ofp, "# Alignment of %d hits satisfying inclusion thresholds saved to: %s\n", msa->nseq, esl_opt_GetString(go, "-A")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
+          }
 	else if (fprintf(ofp, "# No hits satisfy inclusion thresholds; no alignment saved\n") < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
-	  
+
 	esl_msa_Destroy(msa);
       }
 
@@ -862,7 +846,7 @@ int next_block(ESL_SQFILE *sqfp, ESL_SQ *sq, BLOCK_LIST *list, SEQ_BLOCK *block,
 }
 
 /* mpi_master()
- * The MPI version of hmmbuild.
+ * The MPI version of phmmer
  * Follows standard pattern for a master/worker load-balanced MPI program (J1/78-79).
  * 
  * A master can only return if it's successful. 
@@ -903,6 +887,7 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
   int              nquery   = 0;
   int              seed;
   int              textw;
+  int              Aformat  = eslMSAFILE_UNKNOWN;
   int              status   = eslOK;
   int              qstatus  = eslOK;
   int              sstatus  = eslOK;
@@ -935,6 +920,8 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
     dbformat = esl_sqio_EncodeFormat(esl_opt_GetString(go, "--tformat"));
     if (dbformat == eslSQFILE_UNKNOWN) p7_Fail("%s is not a recognized sequence database file format\n", esl_opt_GetString(go, "--tformat"));
   }
+  if ((Aformat  = esl_msafile_EncodeFormat(esl_opt_GetString(go, "--Aformat"))) == eslMSAFILE_UNKNOWN)  // default "Stockholm" is set in OPTIONS
+    p7_Fail("%s is not a recognized MSA file format\n", esl_opt_GetString(go, "--Aformat"));
 
   bg = p7_bg_Create(abc);
 
@@ -1161,8 +1148,7 @@ mpi_master(ESL_GETOPTS *go, struct cfg_s *cfg)
 
 	    esl_msa_FormatAuthor(msa, "phmmer (HMMER %s)", HMMER_VERSION);
 
-	    if (textw > 0) esl_msafile_Write(afp, msa, eslMSAFILE_STOCKHOLM);
-	    else           esl_msafile_Write(afp, msa, eslMSAFILE_PFAM);
+	    esl_msafile_Write(afp, msa, Aformat);
 
 	    if (fprintf(ofp, "# Alignment of %d hits satisfying inclusion thresholds saved to: %s\n", msa->nseq, esl_opt_GetString(go, "-A")) < 0) ESL_EXCEPTION_SYS(eslEWRITE, "write failed");
 	  }
@@ -1467,7 +1453,7 @@ thread_loop(ESL_THREADS *obj, ESL_WORK_QUEUE *queue, ESL_SQFILE *dbfp, int n_tar
         block->count = 0;
         sstatus = eslEOF;
       } else {
-        sstatus = esl_sqio_ReadBlock(dbfp, block, -1, n_targetseqs, FALSE);
+        sstatus = esl_sqio_ReadBlock(dbfp, block, -1, n_targetseqs, /*max_init_window=*/FALSE, FALSE);
         n_targetseqs -= block->count;
       }
 
