@@ -24,12 +24,12 @@
 #include "esl_vectorops.h"
 #include "esl_wuss.h"
 
-#include "cococyk.h"
+#include "cacofold.h"
 #include "contactmap.h"
 #include "correlators.h"
 #include "covariation.h"
 #include "covgrammars.h"
-#include "cykcov.h"
+#include "maxcov.h"
 #include "logsum.h"
 #include "pottsbuild.h"
 #include "ratematrix.h"
@@ -40,18 +40,20 @@ static int    is_allowed_pair(int x, int y, ESL_DMATRIX *allowpair);
 static int    number_pairs(int L, int *ct);
 static int    mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s *mi,
 				double tol, int verbose, char *errbuf);
+static int    mutual_naive_psi(ESL_RANDOMNESS *r, int i, ESL_MSA *msa, struct mutual_s *mi,
+			       double tol, int verbose, char *errbuf);
 static int    mutual_postorder_ppij(int i, int j, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ribosum, struct mutual_s *mi,
 				    ESL_DMATRIX **CL, ESL_DMATRIX **CR, double tol, int verbose, char *errbuf);
 static int    shuffle_col(ESL_RANDOMNESS *r, int nseq, int *useme, int *col, int **ret_shcol, char *errbuf);
 
 int                 
-corr_CalculateCHI(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateCHI(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi; 
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi; 
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
   
   switch (covclass) {
   case C16:
@@ -66,7 +68,11 @@ corr_CalculateCHI(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
     else
       status = corr_CalculateCHI_C16(mi, verbose, errbuf);
     break;
-}
+  default:
+     ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateCHI() CWC not implemented\n");
+     break;
+  }
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateCHI() error\n");
   
   if (verbose) {
     printf("CHI[%f,%f]\n", mi->minCOV, mi->maxCOV);
@@ -76,11 +82,6 @@ corr_CalculateCHI(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
       } 
   }
 
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-
   return status;
 
  ERROR:
@@ -181,13 +182,13 @@ corr_CalculateCHI_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, c
 
 
 int                 
-corr_CalculateOMES(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateOMES(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi;
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
   
   switch (covclass) {
   case C16:
@@ -202,7 +203,11 @@ corr_CalculateOMES(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
     else
       status = corr_CalculateOMES_C16(mi, verbose, errbuf);
     break;
+  default:
+    ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateOMES() CWC not implemented\n");
+    break;
   }
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateOMES() error\n");
   
   if (verbose) {
     printf("OMES[%f,%f]\n", mi->minCOV, mi->maxCOV);
@@ -212,12 +217,7 @@ corr_CalculateOMES(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
       } 
   }
   
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-
-  return status;
+ return status;
   
  ERROR:
   return status;
@@ -314,13 +314,13 @@ corr_CalculateOMES_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose,
 }
 
 int                 
-corr_CalculateGT(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateGT(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi;
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
 
   switch (covclass) {
   case C16:
@@ -329,6 +329,9 @@ corr_CalculateGT(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST *
   case C2:
     status = corr_CalculateGT_C2  (mi, data->allowpair, verbose, errbuf);
     break;
+  case CWC:
+    status = corr_CalculateGT_CWC (mi, data->allowpair, verbose, errbuf);
+    break;
   case CSELECT:
     if (mi->nseq <= mi->nseqthresh || mi->alen <= mi->alenthresh) {
       status = corr_CalculateGT_C2(mi, data->allowpair, verbose, errbuf);
@@ -338,20 +341,16 @@ corr_CalculateGT(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST *
     }
     break;
   }
-  
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateGT() error\n");
+
   if (verbose) {
     for (i = 0; i < mi->alen-1; i++) 
       for (j = i+1; j < mi->alen; j++) {
-	if (data->msamap[i]==15&&data->msamap[j]==70) printf("GT[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
+	  printf("GT[%d][%d] = %f \n", data->msamap[i]+data->firstpos, data->msamap[j]+data->firstpos, mi->COV->mx[i][j]);
       } 
   }
 
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
- 
-  return status;
+ return status;
   
  ERROR:
   return status;
@@ -391,7 +390,46 @@ corr_CalculateGT_C16(struct mutual_s *mi, int verbose, char *errbuf)
       if (gt < mi->minCOV) mi->minCOV = gt;
       if (gt > mi->maxCOV) mi->maxCOV = gt;
     }
+
+   return status;
+}
+
+int                 
+corr_CalculateGT_CWC(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, char *errbuf)
+{
+  double gt;
+  double obs;
+  double exp;
+  int    i, j;
+  int    x, y;
+#if GAPASCHAR
+  int    K = mi->abc->K+1;
+#else
+  int    K = mi->abc->K;
+#endif
+  int    status = eslOK;
+
+  corr_ReuseCOV(mi, GT, C16);
   
+  // GT
+  for (i = 0; i < mi->alen-1; i++) 
+    for (j = i+1; j < mi->alen; j++) {
+      gt  = 0.0;
+      for (x = 0; x < K; x ++)
+	for (y = 0; y < K; y ++) {
+	   if (is_allowed_pair(x,y,allowpair)) {
+	     exp = mi->nseff[i][j] * mi->pm[i][x] * mi->pm[j][y];
+	     obs = mi->nseff[i][j] * mi->pp[i][j][IDX(x,y,K)];
+	     gt += (exp > 0. && obs > 0.) ? obs * log (obs / exp) : 0.0;
+	   }
+	}	  
+      gt *= 2.0;
+
+      mi->COV->mx[i][j] = mi->COV->mx[j][i] = gt;
+      if (gt < mi->minCOV) mi->minCOV = gt;
+      if (gt > mi->maxCOV) mi->maxCOV = gt;
+    }
+
    return status;
 }
 
@@ -453,13 +491,13 @@ corr_CalculateGT_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, ch
 
 
 int                 
-corr_CalculateMI(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateMI(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi;
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
   
   switch (covclass) {
   case C16:
@@ -474,7 +512,11 @@ corr_CalculateMI(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST *
     else
       status = corr_CalculateMI_C16(mi, verbose, errbuf);
     break;
+  default:
+    ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateMI() CWC not implemented\n");
+    break;
   }
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateMI() error\n");
   
   if (verbose) {
     printf("MI[%f,%f]\n", mi->minCOV, mi->maxCOV);
@@ -484,11 +526,6 @@ corr_CalculateMI(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST *
       } 
   }
   
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-  
   return status;
   
  ERROR:
@@ -578,13 +615,13 @@ corr_CalculateMI_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, ch
 
 
 int                 
-corr_CalculateMIr(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateMIr(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi;
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
   
   switch (covclass) {
   case C16:
@@ -599,7 +636,11 @@ corr_CalculateMIr(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
     else
       status = corr_CalculateMIr_C16(mi, verbose, errbuf);
     break;
+  default:
+    ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateMIr() CWC not implemented\n");
+     break;
   }
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateMIr() error\n");
   
   if (verbose) {
     printf("MIr[%f,%f]\n", mi->minCOV, mi->maxCOV);
@@ -609,11 +650,6 @@ corr_CalculateMIr(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
       } 
   }
   
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-  
   return status;
   
  ERROR:
@@ -710,13 +746,13 @@ corr_CalculateMIr_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, c
 
 
 int                 
-corr_CalculateMIg(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateMIg(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi;
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
   
   switch (covclass) {
   case C16:
@@ -731,7 +767,11 @@ corr_CalculateMIg(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
     else
       status = corr_CalculateMIg_C16(mi, verbose, errbuf);
     break;
+  default:
+    ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateMIg() CWC not implemented\n");
+    break;
   }
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateMIg() error\n");
   
   if (verbose) {
     printf("MIg[%f,%f]\n", mi->minCOV, mi->maxCOV);
@@ -741,11 +781,6 @@ corr_CalculateMIg(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
       } 
   }
   
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-  
   return status;
   
  ERROR:
@@ -839,11 +874,11 @@ corr_CalculateMIg_C2(struct mutual_s *mi, ESL_DMATRIX *allowpair, int verbose, c
 }
 
 int                 
-corr_CalculateRAF(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateRAF(COVCLASS covclass, struct data_s *data, ESL_MSA *msa)
 {
-  struct mutual_s *mi = data->mi;
+  struct mutual_s *mi        = data->mi;
   ESL_DMATRIX     *allowpair = data->allowpair;
-  int              verbose = data->verbose;
+  int              verbose   = data->verbose;
   double           psi = 1.0;
   double           cij, qij;
   int              i, j;
@@ -894,29 +929,24 @@ corr_CalculateRAF(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int anal
       } 
   }
   
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-
-  return status;
-  
- ERROR:
   return status;
 }
 
 int                 
-corr_CalculateRAFS(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateRAFS(COVCLASS covclass, struct data_s *data, ESL_MSA *msa)
 {
-  struct mutual_s *mi = data->mi;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
-  ESL_DMATRIX     *bij = NULL;
+  ESL_DMATRIX     *bij     = NULL;
   double           bijs;
   int              i, j;
-  int              status = eslOK;
+  int              status;
 
   corr_ReuseCOV(mi, RAF, C2);
-  corr_CalculateRAF(C2, data, msa, FALSE, NULL, NULL);
+  
+  status = corr_CalculateRAF(C2, data, msa);
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateRAF() error\n");
 
   // RAFS
   bij = esl_dmatrix_Clone(mi->COV);
@@ -926,7 +956,7 @@ corr_CalculateRAFS(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int ana
       
       bijs = 2.0 * bij->mx[i][j];
       if (i > 0 && j < mi->alen-1) bijs += bij->mx[i-1][j+1];
-      if (j > 0 && i < mi->alen-1) bijs += bij->mx[i+1][j-1];
+      if (j > 0 && i < mi->alen-1 && i < j-2) bijs += bij->mx[i+1][j-1];
       bijs *= 0.25;
 
       mi->COV->mx[i][j] = mi->COV->mx[j][i] = bijs;     
@@ -939,15 +969,10 @@ corr_CalculateRAFS(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int ana
     printf("RAFS[%f,%f]\n", mi->minCOV, mi->maxCOV);
     for (i = 0; i < mi->alen-1; i++) 
       for (j = i+1; j < mi->alen; j++) {
-	if (i==5&&j==118) printf("RAF[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
+	printf("RAFS[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
       } 
   }
   
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-
   esl_dmatrix_Destroy(bij);
   return status;
   
@@ -957,16 +982,17 @@ corr_CalculateRAFS(COVCLASS covclass, struct data_s *data, ESL_MSA *msa, int ana
 }
 
 int                 
-corr_CalculateCCF(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist)
+corr_CalculateCCF(COVCLASS covclass, struct data_s *data)
 {
-  struct mutual_s *mi = data->mi; 
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi; 
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
   int              i, j;
-  int              status = eslOK;
+  int              status;
   
   status = corr_CalculateCCF_C16  (mi, verbose, errbuf);
-  
+  if (status != eslOK) ESL_XFAIL(eslFAIL, errbuf, "corr_CalculateCCF() error\n");
+
   if (verbose) {
     printf("CCF[%f,%f]\n", mi->minCOV, mi->maxCOV);
     for (i = 0; i < mi->alen-1; i++) 
@@ -974,13 +1000,7 @@ corr_CalculateCCF(COVCLASS covclass, struct data_s *data, int analyze, RANKLIST
 	if (i==5&&j==118) printf("CCF[%d][%d] = %f \n", i, j, mi->COV->mx[i][j]);
       } 
   }
-
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-
-  return status;
+  return eslOK;
 
  ERROR:
   return status;
@@ -1041,17 +1061,17 @@ corr_CalculateCCF_C16(struct mutual_s *mi, int verbose, char *errbuf)
 }
 
 int                 
-corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int analyze, RANKLIST **ret_ranklist, HITLIST **ret_hitlist, int shiftnonneg)
+corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int shiftnonneg)
 {
-  struct mutual_s *mi = data->mi;
-  char            *errbuf = data->errbuf;
+  struct mutual_s *mi      = data->mi;
+  char            *errbuf  = data->errbuf;
   int              verbose = data->verbose;
-  char            *type = NULL;
+  char            *type    = NULL;
   char            *covtype = NULL;
-  ESL_DMATRIX     *COV  = NULL;
-  double          *COVx = NULL;
-  double           COVavg = 0.0;
-  int              L = mi->alen;
+  ESL_DMATRIX     *COV     = NULL;
+  double          *COVx    = NULL;
+  double           COVavg  = 0.0;
+  int              L       = mi->alen;
   int              i, j;
   int              status = eslOK;
 
@@ -1071,8 +1091,9 @@ corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int analyze, RANK
  
   // COVavg
   for (i = 0; i < L-1; i++) 
-    for (j = i+1; j < L; j++) 
+    for (j = i+1; j < L; j++) {
       COVavg += COV->mx[i][j];
+    }
   if (L > 1) COVavg /= (double)L * ((double)L-1.);
   COVavg *= 2.;
 
@@ -1100,9 +1121,9 @@ corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int analyze, RANK
       else 
 	ESL_XFAIL(eslFAIL, errbuf, "wrong correction type\n");
 
-      if (isnan(mi->COV->mx[i][j])) ESL_XFAIL(eslFAIL, errbuf, "bad covariation\n");
+      if (isnan(mi->COV->mx[i][j])) ESL_XFAIL(eslFAIL, errbuf, "bad covariation COVavg %f\n", COVavg);
 
-      if (mi->COV->mx[i][j] < mi->minCOV) mi->minCOV = mi->COV->mx[i][j];
+      if (mi->COV->mx[i][j] < mi->minCOV) mi->minCOV = mi->COV->mx[i][j]; 
       if (mi->COV->mx[i][j] > mi->maxCOV) mi->maxCOV = mi->COV->mx[i][j];
     }
 
@@ -1121,11 +1142,6 @@ corr_CalculateCOVCorrected(ACTYPE actype, struct data_s *data, int analyze, RANK
       } 
   }
 
-  if (analyze) {
-    status = cov_SignificantPairs_Ranking(data, ret_ranklist, ret_hitlist);
-    if (status != eslOK) goto ERROR;
-  }
-  
   free(type);
   free(covtype);
   esl_dmatrix_Destroy(COV);
@@ -1166,26 +1182,30 @@ corr_Create(int64_t alen, int64_t nseq, int ishuffled, int nseqthresh, int alent
   ESL_ALLOC(mi->nseff,               sizeof(double  *) * alen);
   ESL_ALLOC(mi->ngap,                sizeof(double  *) * alen);
   ESL_ALLOC(mi->pm,                  sizeof(double  *) * alen);
+  ESL_ALLOC(mi->ps,                  sizeof(double  *) * alen);
   for (i = 0; i < alen; i++) {
     ESL_ALLOC(mi->pp[i],             sizeof(double  *) * alen);
     ESL_ALLOC(mi->nseff[i],          sizeof(double   ) * alen);
     ESL_ALLOC(mi->ngap[i],           sizeof(double   ) * alen);
     ESL_ALLOC(mi->pm[i],             sizeof(double   ) * K);
+    ESL_ALLOC(mi->ps[i],             sizeof(double   ) * (abc->K+1));
     for (j = 0; j < alen; j++) {
        ESL_ALLOC(mi->pp[i][j],       sizeof(double  ) * K2);
     }
   }
 
-  mi->COV = esl_dmatrix_Create(alen, alen);
+  mi->COV  = esl_dmatrix_Create(alen, alen);
+  mi->Eval = esl_dmatrix_Create(alen, alen);
  
   /* initialize for adding counts */
   for (i = 0; i < alen; i++) {
-    esl_vec_DSet(mi->pm[i], K, 0.0); 
+    esl_vec_DSet(mi->pm[i], K,        0.0); 
+    esl_vec_DSet(mi->ps[i], abc->K+1, 0.0); 
  
     for (j = 0; j < alen; j++) {
       mi->nseff[i][j] = 0.;
       mi->ngap[i][j]  = 0.;
-      esl_vec_DSet(mi->pp[i][j],       K2, 0.0); 
+      esl_vec_DSet(mi->pp[i][j], K2, 0.0); 
     }
   }
 
@@ -1215,12 +1235,13 @@ corr_Reuse(struct mutual_s *mi, int ishuffled, COVTYPE mitype, COVCLASS miclass)
 
   /* initialize for adding counts */
   for (i = 0; i < mi->alen; i++) {
-    esl_vec_DSet(mi->pm[i], K, 0.0); 
+    esl_vec_DSet(mi->pm[i], K,            0.0); 
+    esl_vec_DSet(mi->ps[i], mi->abc->K+1, 0.0); 
     
     for (j = 0; j < mi->alen; j++) {
       mi->nseff[i][j] = 0.;
       mi->ngap[i][j]  = 0.;
-      esl_vec_DSet(mi->pp[i][j],       K2, 0.0); 
+      esl_vec_DSet(mi->pp[i][j], K2, 0.0); 
     }
   }
 
@@ -1229,13 +1250,15 @@ corr_Reuse(struct mutual_s *mi, int ishuffled, COVTYPE mitype, COVCLASS miclass)
   return eslOK;
 }
 
+
 int
 corr_ReuseCOV(struct mutual_s *mi, COVTYPE mitype, COVCLASS miclass)
 {
   mi->type  = mitype;
   mi->class = miclass;
 
-  if (mi->COV) esl_dmatrix_SetZero(mi->COV);
+  if (mi->COV)  esl_dmatrix_Set(mi->COV,  -eslINFINITY);
+  if (mi->Eval) esl_dmatrix_Set(mi->Eval,  eslINFINITY);
 
   mi->besthreshCOV = -eslINFINITY;
   mi->minCOV       =  eslINFINITY;
@@ -1259,13 +1282,16 @@ corr_Destroy(struct mutual_s *mi)
       free(mi->ngap[i]);
       free(mi->pp[i]);
       free(mi->pm[i]);
+      free(mi->ps[i]);
     }
 
     if (mi->COV)  esl_dmatrix_Destroy(mi->COV);
+    if (mi->Eval) esl_dmatrix_Destroy(mi->Eval);
     free(mi->nseff);
     free(mi->ngap);
     free(mi->pp);
     free(mi->pm);
+    free(mi->ps);
     free(mi);
   }
 }
@@ -1290,6 +1316,24 @@ corr_NaivePP(ESL_RANDOMNESS *r, ESL_MSA *msa, struct mutual_s *mi, double tol, i
   return status;
 }
 
+int 
+corr_NaivePS(ESL_RANDOMNESS *r, ESL_MSA *msa, struct mutual_s *mi, double tol, int verbose, char *errbuf)
+{
+  int64_t alen = msa->alen;
+  int     i;
+  int     status;
+
+  for (i = 0; i < alen; i ++) {
+      status = mutual_naive_psi(r, i, msa, mi, tol, verbose, errbuf);
+      if (status != eslOK) goto ERROR;
+  }
+  
+  return eslOK;
+
+ ERROR:
+  return status;
+}
+
 int
 corr_Marginals(struct mutual_s *mi, double tol, int verbose, char *errbuf)
 {
@@ -1306,11 +1350,13 @@ corr_Marginals(struct mutual_s *mi, double tol, int verbose, char *errbuf)
   for (i = 0; i < mi->alen; i ++) {
     esl_vec_DSet(mi->pm[i], K, 0.0);
 
-    for (j = 0; j < mi->alen; j ++)     
-      for (x = 0; x < K; x ++) {
-	for (y = 0; y < K; y ++) mi->pm[i][x] += mi->pp[i][j][IDX(x,y,K)];
+    for (x = 0; x < K; x ++) {
+      for (j = 0; j < mi->alen; j ++)     
+	for (y = 0; y < K; y ++) {
+	  if (mi->nseff[i][j] > 0) mi->pm[i][x] += mi->pp[i][j][IDX(x,y,K)];
+	}
       }
-
+    
     // normalized
     esl_vec_DNorm(mi->pm[i], K);              
   
@@ -1400,6 +1446,9 @@ corr_Probs(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ri
   default: ESL_XFAIL(eslFAIL, errbuf, "bad method option");
   } 
 
+  status = corr_NaivePS(r, msa, mi, tol, verbose, errbuf);
+  if (status != eslOK) goto ERROR;
+  
   status = corr_Marginals(mi, tol, verbose, errbuf);
   if (status != eslOK) goto ERROR;    
 
@@ -1409,8 +1458,8 @@ corr_Probs(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ri
   if (verbose) {
     for (i = 0; i < mi->alen-1; i ++) {
       for (j = i+1; j < mi->alen; j ++) {
-	if ((i==5&&j==118)||verbose) {
-	  printf("pp[%d][%d] = ", i, j);
+	if (verbose) {
+	  printf("\npp[%d][%d] = ", i, j);
 	  for (x = 0; x < K; x ++) 
 	    for (y = 0; y < K; y ++) {
 	      printf(" %f ", mi->pp[i][j][IDX(x,y,K)]);
@@ -1428,7 +1477,6 @@ corr_Probs(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ri
 	      printf(" %f ",  mi->nseff[i][j]*mi->pp[i][j][IDX(x,y,K)]);
 	    }
 	  printf("\n");
-	  printf("\n");
 	  printf("Ex[%d][%d] = ", i, j);
 	  for (x = 0; x < K; x ++) 
 	    for (y = 0; y < K; y ++) {
@@ -1447,7 +1495,7 @@ corr_Probs(ESL_RANDOMNESS *r, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ri
 
 }
 
-
+ 
 int 
 corr_ValidateProbs(struct mutual_s *mi, double tol, int verbose, char *errbuf)
 {
@@ -1480,6 +1528,16 @@ corr_ValidateProbs(struct mutual_s *mi, double tol, int verbose, char *errbuf)
     }
   }
   
+  /* ps validation */
+  for (i = 0; i < mi->alen; i ++) {
+    status = esl_vec_DValidate(mi->ps[i], mi->abc->K+1, tol, errbuf);
+    if (status != eslOK) {
+      printf("ps[%d]\n", i);
+      esl_vec_DDump(stdout, mi->ps[i], mi->abc->K+1, NULL);
+      ESL_XFAIL(eslFAIL, errbuf, "ps validation failed");
+    }
+  }
+  
   return eslOK;
   
  ERROR:
@@ -1635,8 +1693,7 @@ number_pairs(int L, int *ct)
 
 
 static int    
-mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s *mi,
-		  double tol, int verbose, char *errbuf)
+mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s *mi, double tol, int verbose, char *errbuf)
 {
   int    *coli   = NULL;
   int    *colj   = NULL;
@@ -1653,7 +1710,7 @@ mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s
   int     x, y;
   int     status;
 
-  esl_vec_DSet(mi->pp[i][j], K2, 1e-5); //some prior to avoid zeros
+  esl_vec_DSet(mi->pp[i][j], K2, 1e-10); //some prior to avoid zeros
   mi->nseff[i][j] = 0.;
 
   ESL_ALLOC(coli, sizeof(int) * msa->nseq);
@@ -1668,9 +1725,9 @@ mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s
     resi = coli[s];
     resj = colj[s];
     
-    if (esl_abc_XIsCanonical(msa->abc, resi) && esl_abc_XIsCanonical(msa->abc, resj)) { 
+    if (esl_abc_XIsCanonical(msa->abc, resi) && esl_abc_XIsCanonical(msa->abc, resj)) {
       mi->nseff[i][j]                += msa->wgt[s];
-      mi->pp[i][j][IDX(resi,resj,K)] += msa->wgt[s]; 
+      mi->pp[i][j][IDX(resi,resj,K)] += msa->wgt[s];
     }
 #if GAPASCHAR
     // add the contribution of A - and - - columns
@@ -1690,12 +1747,13 @@ mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s
   }
 
   // normalize
-  esl_vec_DNorm(mi->pp[i][j], K2);             
-
+  esl_vec_DNorm(mi->pp[i][j], K2);
+    
   /* symmetrize */
   for (x = 0; x < K; x ++)
     for (y = 0; y < K; y ++) 
       mi->pp[j][i][IDX(y,x,K)] = mi->pp[i][j][IDX(x,y,K)];
+
   mi->nseff[j][i] = mi->nseff[i][j];
   
   free(coli);
@@ -1713,6 +1771,43 @@ mutual_naive_ppij(ESL_RANDOMNESS *r, int i, int j, ESL_MSA *msa, struct mutual_s
   return status;
 }
 
+static int    
+mutual_naive_psi(ESL_RANDOMNESS *r, int i,  ESL_MSA *msa, struct mutual_s *mi, double tol, int verbose, char *errbuf)
+{
+  int    *coli   = NULL;
+  int    *shcoli = NULL;
+  int     K = mi->abc->K+1;
+  int     s;
+  int     resi;
+  int     status;
+
+  esl_vec_DSet(mi->ps[i], K, 1e-5); //some prior to avoid zeros
+
+  ESL_ALLOC(coli, sizeof(int) * msa->nseq);
+  for (s = 0; s < msa->nseq; s ++) 
+    coli[s] = msa->ax[s][i+1];
+
+  /* consider gaps too */
+  for (s = 0; s < msa->nseq; s ++) {
+    resi = coli[s];
+    if (resi < K) mi->ps[i][resi] += msa->wgt[s];
+  }
+
+  // normalize
+  esl_vec_DNorm(mi->ps[i], K);
+  status = esl_vec_DValidate(mi->ps[i], K, tol, errbuf);
+
+  free(coli);
+  if (shcoli) free(shcoli);
+  return eslOK;
+
+ ERROR:
+  if (coli)   free(coli);
+  if (shcoli) free(shcoli);
+
+  return status;
+}
+
 
 int 
 mutual_postorder_ppij(int i, int j, ESL_MSA *msa, ESL_TREE *T, struct ribomatrix_s *ribosum, struct mutual_s *mi,
